<?php

namespace App;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;

class CartRequest extends Model {

	const SAVE=0;
	const UPDATE=1;
	public static $last_order = null;
	protected $table = 'requests';
	public $timestamps = false;
	protected $fillable = [ 'client_id', 'shop_id', 'total', 'status', 'no_bill', 'lon', 'lat', 'user_id', 'order_no','estimated_to_arrive', 'start_time', 'feedback', 'rate', 'delivery_rate', 'product_rate', 'mobile', 'phone' ];

	protected $appends = [ 'status_display', 'human_date'  ];



	public function getHumanDateAttribute( $value ) {
		Carbon::setLocale( 'ar' );
		if ( is_null( $this->created_at ) ) {
			return $value;
		}
		$value = Carbon::parse($this->created_at)->diffForHumans( Carbon::now() );

		return $value;
	}

	public function requestDetails() {
		return $this->hasMany( 'App\RequestDetails', 'request_id', 'id' );
	}

	public function client() {
		return $this->belongsTo( 'App\Client' );
	}


	public function shop() {
		return $this->belongsTo( 'App\Badrshop', 'shop_id', 'serial_id' );
	}

	public function getStatusDisplayAttribute() {
		$status = array(
			"0" => "ملغى",
			"1" => "جارى التنفيذ",
			"3" => "جاري الشحن",
			"2" => "تم التسليم",
            '4' => ' معلق لحين اكمال الدفع',
            '5' => 'معلق للتجهيز من قبل الادارة',
		);
		if(is_null($this->attributes['status'])){
			return ;
		}
		logger($this->attributes['status']);
		if ($this->attributes['status'] >= 6)
			return '--';
		return $status[ $this->attributes['status'] % 6 ];
	}
	public function saleProcess() {
		return $this->belongsTo( 'App\SaleProcess','no_bill' );
	}

	public static function getNextOrderIndex($action_type, $shop_id, $order_id=null) {
		$last_order = null;
		if (is_null(self::$last_order)) {
			if ($action_type == self::SAVE) {
				$last_order = CartRequest::where('shop_id', $shop_id)
					->select('order_no')
					->orderBy('id', 'DESC')
					->first();
				if (is_null($last_order)) {
					$last_order = 1;
				} else {
					$last_order = $last_order->order_no + 1;
				}
			} else {
				$last_order = CartRequest::where('shop_id', $shop_id)
					->where('id', $order_id)
					->select('order_no')
					->first();
	
				$last_order = $last_order->order_no;
			}
		}
		self::$last_order = $last_order;
		return $last_order;
	}


}
