<?php

namespace App\Http\Controllers\ApiV2;

use App\Badrshop;
use App\BillAdd;
use App\Http\Controllers\ApiV2\Services\RequestService;
use App\LogSystem;
use App\BillAddHistory;
use App\Card;
use App\CartRequest;
use App\Client;
use App\ClientNotification;
use App\Coupon;
use App\Http\Controllers\Controller;
use App\Item;
use App\ItemUnit;
use App\RequestDetails;
use App\RequestSettings;
use App\ShippingDistancesPrice;
use Illuminate\Http\Request;

use App\Http\Requests;

class NewOrderController extends BasicController
{
    protected RequestService $requestService;
    public function __construct(Request $request, RequestService $requestService)
    {
        $this->requestService = $requestService;
        app()->setLocale($request->header('Accept-Language') ?? 'ar');
    }

    private function checkAvailability($orderData, $shop_id, $order_id=null)
    {
        $available_items = $not_available_items = $available_cards = [];
        $x = 0;
        logger($orderData);
        if (is_array($orderData) && sizeof($orderData) > 0) {
            foreach ($orderData as $it) {
                $item_id = $it['item_id'];
                $quantity = $it['quantity'];
                $unit_id = $it['unit_id'];
                $item = Item::find($item_id);

                if ($item->unit_id != $unit_id) {
                    $unit = ItemUnit::where('shop_id', $shop_id)->where('unit_id', $unit_id)->where('item_id', $item_id)->first();
                    $quantity =  ($unit->unit_value ?? 1) * $quantity;
                }

                $settings = $this->getOrderSettings($shop_id);
                $max_count = $settings->max_items;

                if (!in_array($item->card_company_id, ['', '0', null])) {
                    $max_count = $settings->max_cards;
                    $card_count = $this->cardCount($item);

                    if ($card_count < $max_count) {
                        $max_count = $card_count;
                    }
                } else {
                    if ($settings->dis_req_quantity == 1) {
                        $getRequests = \DB::table('request_details')
                            ->join('requests', function ($q) use ($item) {
                                $q->on('request_details.request_id', '=', 'requests.id')
                                    ->where('request_details.item_id', '=', $item->id)
                                    ->where('requests.client_id', '=', auth('client')->id());
                            })
                            ->leftJoin('items_unit', function ($query) {
                                $query->on(function ($q) {
                                    $q->on('items_unit.id', '=', 'request_details.unit_id')
                                        ->orOn('items_unit.unit_id', '=', 'request_details.unit_id');
                                })->on('items_unit.item_id', '=', 'request_details.item_id');
                            })
                            ->where('requests.no_bill', null)
                            ->whereNotIn('requests.status', ['0'])
                            ->selectRaw('SUM(IF(items_unit.id > 0,items_unit.unit_value * quantity,quantity))AS quantity')
                            // ->latest('id')
                            ->first()->quantity;

                        $minus_qty_from_order = 0;

                        if ($order_id > 0) {
                            $detailsReduced  = RequestDetails::
                            leftJoin('items_unit', function ($q) {
                                $q->on('items_unit.unit_id', '=', 'request_details.unit_id');
                                $q->on('items_unit.item_id', '=', 'request_details.item_id');
                            })
                            ->where('request_id', $order_id)
                            ->where('request_details.item_id', $item_id)
                            ->where('request_details.unit_id', $unit_id)
                            ->selectRaw(' if( items_unit.id > 0, items_unit.unit_value * quantity ,quantity ) as quantity ')
                            ->first();
                            // end of edits

                            $minus_qty_from_order = $detailsReduced->quantity ?? 0;
                        }

                        $getRequests = $getRequests - $minus_qty_from_order;
                    } else {
                        $getRequests = 0;
                    }

                    $newQty = $item->quantity - $getRequests;

                    if ($newQty <= 0) {
                        $max_count = 0;
                    }

                    if ($newQty < $max_count && $newQty > 0) {
                        $max_count = $newQty;
                    }
                }
        
                if ($item && $item->quantity >= $quantity) {
                    $available_items[] = '1';
                    $is_com = $this->cardCheck($item);

                    if ($is_com != '0') {
                        $required_qty = $quantity;

                        $cards_query = Card::where([
                            'shop_id' => $item->shop_id,
                            'item_id' => $item_id,
                            'card_state' => 0,
                        ]);

                        $saved_qty = $cards_query->count();

                        if ($required_qty > $saved_qty) {
                            $neededQty = $required_qty - $saved_qty;

                            if ($neededQty > 0) {
                                $available_cards[] = '1';
                            }
                        }
                    }
                } elseif($item) {
                    $not_available_items[] = [
                            'item_id'  => $item_id,
                            'quantity' => $quantity,
                            'unit_id'  => $unit_id,
                            'available'=> $item->quantity
                        ];
                }
            }
        }
        
        if ( count($not_available_items) > 0 ) {
            return $not_available_items;
        }
        
        \Log::info('yyyyyyyyyyyyy', [$x, 8888888]);

        if($x == '2'){
            return '2';
        }
        
        if (sizeof($available_items) == sizeof($orderData) && sizeof($available_cards) == 0) {
            return 'yes';
        }
        return 'no';
    }

    private function getOrderSettings($shop_id)
    {
        $settings = RequestSettings::where('shop_id', $shop_id)->first();
        if (is_null($settings)) {
            $settings = new RequestSettings();
            $settings->fee = 0;
            $settings->min_purchase = 0;
            $settings->max_charge = 0;
            $settings->max_cards = 5;
            $settings->max_items = 20;
            $settings->shop_id = $shop_id;
            $settings->save();
        }

        return $settings;
    }
    
    public function saveRequest(Request $request, $id)
    {
        $validator = \Validator::make($request->all(), [
            'phone' => 'required'
        ]);
        
        if ($validator->fails())
            return [
                'qty_problem' => false,
                'error' => $validator->errors(),
                'status' => false
            ];

        if (auth('client')->user()) {
            $settings = RequestSettings::where('shop_id', $id)->first();
            $shop = Badrshop::where('serial_id', $id)->firstOrFail();
            $new_details = $request->order_details;
    
            if ($request->has('order_id') && $request->order_id > 0) {
    
                $order_id = $request->order_id;
    
                $currentorder = CartRequest::find($order_id);
    
                if ($currentorder->status != 1) {
                    return response()->json(['qty_problem' => false, 'status' => false, 'msg' => 'لا يمكن التعديل لان تم تأكيد الطلب من قبل الإدارة']);
                }
            }
            // logger($currentorder);
            // dd('test');
            
             if (is_array($new_details) && sizeof($new_details) <= 0) {
                 
                return response()->json(['qty_problem' => false, 'status' => false, 'msg' => 'لا يمكن حفظ الطلب - السلة فارغة']);
        
             }
             
             
            if ($settings->dis_req_quantity == 1) {
                $avilable = $this->checkAvailability($new_details, $id, $request->order_id ?? null);
               
                \Log::info("request 4 : " . json_encode([ $avilable, $request->all()]));
                
                if (is_array($avilable) && count($avilable) > 0) {
                    return response()->json(['qty_problem' => true, 'status' => false, 'msg' => 'هناك اصناف ليس لها كميات متاحة', 'items' => $avilable]);
                }

               if($avilable == '2'){
                        $settings = $this->getOrderSettings($id);
                        $max = $settings->max_items;
                        return response()->json(['qty_problem' => true, 'status' => false, 'msg' => 'تم تجاوز الحد الاقصي للكمية -  ' .  $max  ]);
                    }
                    
                    
                if ($avilable != 'yes') {
                    
                    return response()->json(['qty_problem' => true, 'status' => false, 'msg' => 'لا يمكن حفظ الطلب لعدم توافر الكمية']);
                }
            }
    
            $type = $request->payment_type;
    
            $order = $this->purchaseOrder($id, $request, $type, $order_id ?? null);
            
            // dd($order);
            if ($order != '0') {
    
                $infouser  = Client::where('id', $request->client_id)->first();
                $content   = 'تم تسجيل طلبك رقم  ' . $order[0] . ' بنجاح';
                $player_id = $infouser->player_id;
                $tokens    = [$player_id];
    
                $headings = [
                    'en' => $shop->shop_name,
                    'ar' => $shop->shop_name,
                ];
                
                $data                           = ['order_id' => $order, 'type' => 1];
                $client_notification            = new ClientNotification();
                $client_notification->client_id = $infouser->id;
                $client_notification->order_id  = $order[1];
                $client_notification->message   = $content;
                $client_notification->type      = 1;
                $client_notification->shop_id   = $id;
                $client_notification->save();
                
                $msg = trans('theme.order_success') . " - " . trans('theme.order_no') . ":" . $order[0];
    
                $infouser->save();
                $response = [
                    'qty_problem' => false,
                    'status'    => true,
                    "msg"       => $msg,
                    'order_id'  => $order[1],
                    'final_net' => price_decimal($order[2], $id)
                ];
                \Log::info("order number {$order[0]} Request: " . json_encode($request->all()));
                return response()->json($response, 201);
            } else {
                return response()->json([
                    'qty_problem' => false,
                    'status' => false,
                    'msg'    => trans('theme.order_fail')
                ]);
            }
        }
        else
        {
             return response()->json([
                'qty_problem' => false,
                'status' => false,
                'msg'    => 'unAuthorized'
            ]);
           
        }
        
       
    }

    protected function purchaseOrder($id, $request, $pay_type, $order_id, $orderDetails = null) {
        try {
            \Log::info( $request->getUri() );
            \Log::info( json_encode($request->all()) );
            $process = \DB::transaction(function () use ($id, $request, $pay_type , $order_id, $orderDetails) {

                /*** kareem update 03-03-2024 ***/
                $service = $this->requestService
                    ->setShop($id)
                    ->setBillAdds()
                    ->setOrder($order_id)
                    ->setOrderIndex()
                    ->setOrderDetails()
                    ->setRequestSettings()
                    ->setRequest($request)
                    ->setRequestDetails($orderDetails);
                $settings = $service->getRequestSettings();
                $order_data = $service->getOrderData();
                
                // dd($order_data);
                /*** end of updates ***/

                $total = 0;
                $discount = 0;
                $couponDiscount = 0;
                $vat = 0;
                $net = 0;
                $all_fee = 0;

                foreach ($order_data as $orderItem){
                    $total += ($orderItem['total_main'] * $orderItem['quantity']);
                    $discount += ($orderItem['discount'] * $orderItem['quantity']);
                    $vat += $orderItem['vat'];
                }
                
                $coupon = Coupon::where('id', $request->coupon_id)->first();
                $coupon_val = 0;
                if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0'])) {
                    if ($coupon && $coupon->check()) {
                        if ($coupon->discount_type == 1) {
                            $coupon_val = $coupon->discount_value;
                        } else {
                            $coupon_val = ($total - $discount) * ($coupon->discount_value / 100);
                        }
                    }
                }
                $couponDiscount = $coupon_val;

                $net = $total - ( $discount + $couponDiscount ) + $vat;
                logger(['total' => $total, 'discount' => $discount, 'couponDiscount' => $couponDiscount, 'vat' => $vat, 'net' => $net]);


                $all_fee = $this->CalcFee($request, $net);

                if ($net > $settings->max_charge) {
                    $all_fee = 0;
                }

                if ($request->has('fast_charge') && !in_array($request->fast_charge, ['', null]) && $request->fast_charge == '1' && $settings) {

                    if ($settings->fee_type == 0) {
                        $all_fee = $settings->fee / 100 * $net;
                    }
                }

                $status = 1;
                if ($pay_type == 1) {
                    $status = 4;
                }


                $requestc = ($service->isSaveRequest()) ? new CartRequest() : CartRequest::find($order_id);
                $requestc->client_id = $request->client_id ?? 0;
                $requestc->shop_id = $id;
                $requestc->status = $status;
                $requestc->lon = $request->lat;
                $requestc->lat = $request->lon;
                $requestc->total = $total;
                $requestc->net = $net + $all_fee;
                $requestc->fee = $all_fee;
                $requestc->order_no = $service->getOrderIndex();
                $requestc->fee_type = $settings->fee_type;
                $requestc->client_name = $request->client_name;
                $requestc->email = $request->email;
                $requestc->mobile = $request->mobile;
                $requestc->phone = $request->phone;
                $requestc->payment_type = $pay_type;
                $requestc->city_id = $request->city_id ?? null;
                $requestc->coupon_id = $request->coupon_id ?? 0;
                $requestc->discount = $discount;
                $requestc->address = $request->address;
                $requestc->created_at = date('Y-m-d H:i:s');
                $requestc->updated_at = date('Y-m-d H:i:s');
                $requestc->save();
                
                
                if ($order_id > 0) {
                    $current_details = RequestDetails::where('request_id', $order_id)->get();
                    foreach ($current_details as $cd) {
                        $cd->delete();
                    }
                }

                $order_total = 0;
                foreach ($order_data as $data) {
                    $it = Item::find($data['item_id']);

                    $keys = [
                        'item_id'    => $data['item_id'],
                        'unit_id'    => $data['unit_id'],
                    ];
                    $values = [
                        'item_id'    => $data['item_id'],
                        'unit_id'    => $data['unit_id'],
                        'quantity' => $data['quantity'],
                        'item_notes' => $data['item_notes'],
                        'price' => $data['total_after_discount'],
                        'sale_price' => $data['o_price'],
                        'shop_id' => $id,
                        'discount_percent' => $data['discount_percent'],
                    ];
                    $requestc->requestDetails()->create($values);
                    if ($it->vat_state != 0)
                        $order_total += $data['total_after_coupon'];
                }

                if ($service->isSaveRequest()) {
                    foreach ($service->getBillAdds() as $iadd) {
                        if ($iadd->check_addition) {
                            $vati = $iadd->addition_value;
                        } else {
                            $vati = $iadd->addition_value / 100 * $order_total;
                        }

                        $bills_add_history = new BillAddHistory();
                        $bills_add_history->bill_id = 0;
                        $bills_add_history->addition_id = $iadd->id;
                        $bills_add_history->addition_value = $vati;
                        $bills_add_history->shop_id = $id;
                        $bills_add_history->type = 1;
                        $bills_add_history->request_id = $requestc->id;
                        $bills_add_history->save();
                    }
                } else {
                    $bills_add_histories = BillAddHistory::where('request_id', $requestc->id)->where('shop_id', $id)->get();

                    foreach ($bills_add_histories as $index => $bills_add_history) {

                        $vati = $service->getBillAdds()[$index]->addition_value / 100 * $order_total;

                        $bills_add_history->bill_id = 0;
                        $bills_add_history->addition_id = $service->getBillAdds()[$index]->id;
                        $bills_add_history->addition_value = $vati;
                        $bills_add_history->shop_id = $id;
                        $bills_add_history->type = 1;
                        $bills_add_history->request_id = $requestc->id;
                        $bills_add_history->save();
                    }
                }

                return [$requestc->order_no, $requestc->id, $requestc->net];
            });

            \Log::info('Order Created Successfully , Order No : ' . $process[0]);
            return $process;
        } catch (\Exception $e) {
            \Log::error('Error in purchaseOrder method: ' . $e->getMessage() . ' Line: ' . $e->getLine());
            LogSystem::create([
                'user_id'    => auth()->id() ?? 1,
                'date_event' => date('Y-m-d H:i:s'),
                'event'      => 'Error in purchaseOrder method: ' . $e->getMessage() . ' Line: ' . $e->getLine(),
                'shop_id'    => $id,
                'type'       => 'purchaseOrder',
            ]);
            return 'Error in purchaseOrder method: ' . $e->getMessage() . ' Line: ' . $e->getLine();
        }
    }

    protected function purchaseOrderOld03_03_2024($id, $request, $pay_type, $order_id, $orderDetails = null)
    {

        try {
            $process = \DB::transaction(function () use ($id, $request, $pay_type , $order_id, $orderDetails) {

                $shop = Badrshop::where('serial_id', $id)->firstOrFail();
                $bills_add_ = BillAdd::where('shop_id', $id)->get();

                // $last_order = CartRequest::where('shop_id', $id)->select('order_no')
                //     ->orderBy('id', 'DESC')->first();
                // if (is_null($last_order)) {
                //     $last_order = 1;
                // } else {
                //     $last_order = $last_order->order_no + 1;
                // }

                if (in_array($order_id, [null, 0, '0'])) {
                    $last_order = CartRequest::where('shop_id', $id)
                        ->select('order_no')
                        ->orderBy('id', 'DESC')
                        ->first();
                    if (is_null($last_order)) {
                        $last_order = 1;
                    } else {
                        $last_order = $last_order->order_no + 1;
                    }
                } else {
                    $last_order = CartRequest::where('shop_id', $id)
                        ->where('id', $order_id)
                        ->select('order_no')
                        ->first();

                    $last_order = $last_order->order_no;
                }
//                return ['sss', $last_order];
                $settings = RequestSettings::where('shop_id', $id)->first();
                # الحد الاقصي لفرض الرسوم
                // $fee = $settings->fee;
                // $fee_type = $settings->fee_type;
                // $max_charge = $settings->max_charge;
                $items = $orderDetails ?? $request->order_details;

                $addres_lat = $request->lat ?? 0;
                $addres_lon = $request->lon ?? 0;

                $order_data = [];

                $total = 0;
                $discount = 0;
                $couponDiscount = 0;
                $vat = 0;
                $net = 0;
                $all_fee = 0;

//                return ['aaaa', $items[0]];

                $not_cards = [];
                $coupon = Coupon::where('id', $request->coupon_id)->first();


                $x = [];
                $details = RequestDetails::where('request_id',$order_id)->where('shop_id', $request->get('shop_id'))->get();
                foreach ($items as $it) {

                    $item_id = $it['item_id'];
                    $quantity = $it['quantity'];
                    $unit_id = $it['unit_id'];
                    $item_notes = $it['item_notes'] ?? null;
                    $item = Item::find($item_id);
                    $isCard = $this->cardCheck($item);

                    //dd($item->unit_id,  $unit_id);

                    if ($item->unit_id == $unit_id) {
                        if($order_id != null) {
                            $request_details=RequestDetails::where('request_id',$order_id)->where('item_id',$item->id)->first();

                            if($request_details != null) {
                                if($request_details->quantity == $quantity && $request_details->unit_id == $unit_id) {
                                    $price = $request_details->sale_price;
                                } else {
                                    $price = $item->sale_price;
                                }
                            } else {
                                $price = $item->sale_price;
                            }

                        } else {
                            $price = $item->sale_price;
                        }


                    } else {
                        $unit = ItemUnit::where('shop_id', $id)->where('unit_id', $unit_id)->where('item_id', $item_id)->first();
                        $price = $unit->unit_price;
                    }

                    $price_original = $price;

                    $price_new = $price;
                    $dis = 0;
                    
                    if ($item->withDiscount) {
                        if (!in_array($order_id, [0, null, '0'])) { // لو تعديل اوردر
                            if (!$details->where('item_id', $item_id)->count()) { // لو مش موجود في الاوردر
                                // $dis = $item->discount_percent / 100 * $price_original * $quantity;
                                // $price_new = (1 - ($item->discount_percent / 100)) * $price_original;
                                $price_new = $price_original - $item->discount_amount;
                                /* if ($item->vat_state == 2) {
                                    $price_new = (1 - ($item->discount_percent / 100)) * $price;
                                }*/
                            }
                        } else {
                            $price_new = $price_original - $item->discount_amount;
                        }
                    }

                    $coupon_discount = 0;
                    $price = $price_new;
                    $price_original = $price_new;
                    if ($item->vat_state == 2) {
                        $v = \DB::table('bills_add')
                            ->where('shop_id', $id)
                            ->where('id', $item->vat_id)->first();


                        if ($v) {
                            if ($v->check_addition == 0) {
                                $price_original = $price_original / (1 + $v->addition_value / 100);
                            }
                            //                        $price_original = price_decimal($price_original, $id);
                        }
                    }

                    $for_v =0;
                    $price_new = $price_original;
                    $item_total_price = $price_new * $quantity;
                    $item_total_price_main = $price * $quantity;
                    $price_new_vat = $item_total_price;

                    if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0'])) {


                        if ($coupon && $coupon->check()) {
                            if ($coupon->discount_type == 0) {
                                $coupon_discount = $coupon->discount_value * $quantity *  (  $price_new) / 100;
                                $coupon_value = $coupon_discount;
                                $price_new_vat =$item_total_price - $coupon_discount;
                            }/* else {

                                $coupon_discount = ($coupon->discount_value / $item_total_price) ;
                                $coupon_value = $item_total_price * $coupon_discount;
                                $price_new_vat =$item_total_price - $coupon_value;
                                //  dd([$coupon_discount,$price_new_vat,$item_total_price,$coupon_value]);
                            }*/
                        }
                    }


                    // $x[] = $coupon_value;

                    //                $total_net = price_decimal(($item->sale_price) * $quantity, $id);
//                    $discount = $discount + $dis;
//                    return ['refwsfsdfsf', rand(), $price_new_vat];
                    $for_v += $price_new_vat;

                    $total += $item_total_price;


                    $item_vat = 0;

                    if ($item->vat_state == 1) {
                        foreach ($bills_add_ as $add) {
                            if ($add->check_addition) {
                                $item_vat = $add->addition_value;
                                $vat += $item_vat;
                                $net += $for_v + $add->addition_value;
                            } else {
                                $item_vat = $for_v * $add->addition_value / 100;
                                $vat += $item_vat;
                                $net += $for_v + ($for_v * $add->addition_value / 100);
                            }
                        }
                    } else {
                        $net += $for_v;
                    }

                    $im_fee = 0;

                    // if ($isCard == '0') {
                        $not_cards[] = '1';
                        $im_fee = $settings->fee;
                        if ($settings->fee_type == '0') {
                            $im_fee = $im_fee / 100 * ($item_total_price_main - $discount);
                        }
                    // }
                    $all_fee += $im_fee;


                    /************ By Mahmoud Arafat 15-06-2023 **********/
                    /*
                    $order_data[] = [
                        'item_id' => $item_id,
                        'unit_id' => $unit_id,
                        'item_notes' => $item_notes,
                        'quantity' => $quantity,
                        'o_price' => $price_original,
                        'price' => $price_new,
                        'is_card' => $isCard,
                        'discount' => $dis,
                        'vat' => $item_vat,
                        'total' => $item_total_price,
                        'fee' => $request->fee
                    ];
                    */

                    $order_data[] = [
                        'item_id' => $item_id,
                        'unit_id' => $unit_id,
                        'item_notes' => $item_notes,
                        'quantity' => $quantity,
                        'o_price' => $price,
                        'price' => $price_new,
                        'is_card' => $isCard,
                        'discount' => $dis,
                        'coupon_discount' => $coupon_discount,
                        'vat' => $item_vat,
                        'total_main' => $item_total_price_main,
                        'total_after_discount' => $item_total_price,
                        'total_after_coupon' => $item_total_price - $coupon_discount,
                        'fee' => $request->fee,
                        'for_vat' => $for_v
                    ];
                    /*************************************************/
                }


                $coupon_val = 0;
                if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0'])) {


                    if ($coupon && $coupon->check()) {
                        if ($coupon->discount_type == 1) {

                            $coupon_val = $coupon->discount_value;
                        }
                    }}

                //   dd($x);

//                return [111, $order_data];
                /************ By Mahmoud Arafat 15-06-2023 **********/
                $total = 0;
                $discount = 0;
                $couponDiscount = 0;
                $vat = 0;
                $net = 0;
                $all_fee = 0;
//                $net = $total + $vat;
                foreach ($order_data as $orderItem){
                    $total += $orderItem['total_main'];
                    $discount += $orderItem['discount'];
                    $couponDiscount += $orderItem['coupon_discount'];
                    $vat += $orderItem['vat'];
                }

                // $net = $total - ( $discount + $couponDiscount ) + $vat;

                /* new update ***********/

                $net = $total - ( $discount + $couponDiscount ) + $vat;

                $coupon_discount2 = ($coupon_val / $net) ;

                $vat = $vat * (1 - $coupon_discount2);
                $net = $net * (1 - $coupon_discount2);


                /*********************/
//                return [$net, 444];
                // $discount = $discount + $couponDiscount ;
                $discount = $discount + $couponDiscount + $coupon_val;
                /****************************************************/


                $all_fee = $this->CalcFee($request, $net);
                // if(sizeof($not_cards) > 0){
                //      if ($settings->fee_type == 0) {
                //          $all_fee = $net * $settings->fee / 100;
                //      } else {
                //          $all_fee = $settings->fee;
                //      }

                     if ($net > $settings->max_charge) {
                         $all_fee = 0;
                     }
                //  }

                //            $all_fee = $request->fee;

                //            $all_fee =$this->CalcFee($request);


                if ($request->has('fast_charge') && !in_array($request->fast_charge, ['', null]) && $request->fast_charge == '1' && $settings) {

                    if ($settings->fee_type == 0) {
                        $all_fee = $settings->fee / 100 * $net;
                    }
                }

                $status = 1;
                if ($pay_type == 1) {
                    $status = 4;
                }

//              return [44444, $net, $all_fee];
                /*********** By Mahmoud Arafat 15-06-2023 ***************/
                // $discount += $coupon_discount;
                /********************************************************/


                $requestc = (in_array($order_id, [null, 0, '0'])) ? new CartRequest() : CartRequest::find($order_id);
                // $requestc = new CartRequest();
                $requestc->client_id = $request->client_id ?? 0;
                $requestc->shop_id = $id;
                $requestc->status = $status;
                $requestc->lon = $addres_lat;
                $requestc->lat = $addres_lon;
                $requestc->total = $total;
                /************ By Mahmud Arafat 15-06-2023 *****************/
//                $requestc->net = $net + $all_fee - $coupon_discount;
                $requestc->net = $net + $all_fee;
                /**********************************************************/
                $requestc->fee = $all_fee;
                $requestc->order_no = $last_order;
                $requestc->fee_type = $settings->fee_type;
                $requestc->client_name = $request->client_name;
                $requestc->email = $request->email;
                $requestc->mobile = $request->mobile;
                $requestc->phone = $request->phone;
                $requestc->payment_type = $pay_type;
                $requestc->city_id = $request->city_id ?? null;
                $requestc->coupon_id = $request->coupon_id ?? 0;

                /* if ($pay_type == 2) {
                     if ($request->hasFile('trans_file')) {

                         $file = $request->file('trans_file');
                         $ext = strtolower($file->getClientOriginalExtension());

                         $file_name = 'trans_' . date("Y-m-d") . '_' . time() . '.' . $ext;

                         $destinationPath = '../c-admin/upload/trans_files';

                         $file->move($destinationPath, $file_name);
                         $requestc->trans_img = $file_name;
                     }
                 }*/

                $requestc->discount = $discount;

                $requestc->address = $request->address;

                $requestc->created_at = date('Y-m-d H:i:s');
                $requestc->updated_at = date('Y-m-d H:i:s');

                $requestc->save();



                $order_total = 0;

                foreach ($order_data as $data) {
                    $it = Item::find($data['item_id']);

                    //                if ($pay_type == '1') {
                    $cardCheck = $data['is_card'];
                    if ($cardCheck != '0') {
                        $cards = $this->getCards($it, $data['quantity']);
                        //dd( $data['quantity'], count($cards), $cards);
                        foreach ($cards as $card) {
                            $card->request_id = $requestc->id;
                            $card->card_state = 1;
                            $card->save();
                        }
                    }
                    //                }

                    $requestc->requestDetails()->updateOrCreate([
                        'item_id' => $data['item_id'],
                        'unit_id' => $data['unit_id'],
                    ],
                    [
                        'quantity' => $data['quantity'],
                        'item_notes' => $data['item_notes'],
                        'price' => $data['price'],
                        'sale_price' => $data['o_price'],
                        'shop_id' => $id,
                    ]);
//                    $details->request_id = $requestc->id;
//                    $details->item_id = $data['item_id'];
                    //                $details->size_id = $item->attributes['size'];
                    //                $details->color_id = $item->attributes['color'];
//                    $details->unit_id = $data['unit_id'];
//                    $details->quantity = $data['quantity'];
//                    $details->item_notes = $data['item_notes'];
//                    $details->price = $data['price'];

                    /************ By Mahmud Arafat 15-06-2023 *****************/
//                    $details->sale_price = $it->sale_price;
//                    $details->sale_price = $data['o_price'];
                    /**********************************************************/


                    //              $details->original_price = $it->sale_price;
//                    $details->shop_id = $id;
                    //dd($details->size_id ,$details->color_id);
//                    $details->save();
                    if ($it->vat_state != 0)
                        /************ By Mahmud Arafat 15-06-2023 *****************/
//                        $order_total += $data['price'] * $details->quantity;
                        $order_total += $data['total_after_coupon'];
                    /***********************************************************/
                }


                //            dd($order_total);
                // foreach ($bills_add_ as $iadd) {
                //     if ($iadd->check_addition) {
                //         $vati = $iadd->addition_value;
                //     } else {
                //         $vati = $iadd->addition_value / 100 * $order_total;
                //     }
                //     $bills_add_history = new BillAddHistory();
                //     $bills_add_history->bill_id = 0;
                //     $bills_add_history->addition_id = $iadd->id;
                //     $bills_add_history->addition_value = $vati;
                //     $bills_add_history->shop_id = $id;
                //     $bills_add_history->type = 1;
                //     $bills_add_history->request_id = $requestc->id;
                //     $bills_add_history->save();
                // }

                if (in_array($order_id, [null, 0, '0'])) {
                    foreach ($bills_add_ as $iadd) {
                        if ($iadd->check_addition) {
                            $vati = $iadd->addition_value;
                        } else {
                            $vati = $iadd->addition_value / 100 * $order_total;
                        }

                        $bills_add_history = new BillAddHistory();
                        $bills_add_history->bill_id = 0;
                        $bills_add_history->addition_id = $iadd->id;
                        $bills_add_history->addition_value = $vati;
                        $bills_add_history->shop_id = $id;
                        $bills_add_history->type = 1;
                        $bills_add_history->request_id = $requestc->id;
                        $bills_add_history->save();
                    }
                } else {
                    $bills_add_histories = BillAddHistory::where('request_id', $requestc->id)->where('shop_id', $id)->get();

                    foreach ($bills_add_histories as $index => $bills_add_history) {

                        $vati = $bills_add_[$index]->addition_value / 100 * $order_total;

                        $bills_add_history->bill_id = 0;
                        $bills_add_history->addition_id = $bills_add_[$index]->id;
                        $bills_add_history->addition_value = $vati;
                        $bills_add_history->shop_id = $id;
                        $bills_add_history->type = 1;
                        $bills_add_history->request_id = $requestc->id;
                        $bills_add_history->save();
                    }
                }

                return [$requestc->order_no, $requestc->id, $requestc->net];

                //            return $data;

            });

            \Log::info('Order Created Successfully , Order No : ' . $process[0]);
            return $process;
        } catch (\Exception $e) {
            \Log::error('Error in purchaseOrder method: ' . $e->getMessage() . ' Line: ' . $e->getLine());
            LogSystem::create([
                'user_id'    => auth()->id() ?? 1,
                'date_event' => date('Y-m-d H:i:s'),
                'event'      => 'Error in purchaseOrder method: ' . $e->getMessage() . ' Line: ' . $e->getLine(),
                'shop_id'    => $id,
                'type'       => 'purchaseOrder',
            ]);
            return 'Error in purchaseOrder method: ' . $e->getMessage() . ' Line: ' . $e->getLine();
        }
        /*   } catch (\Exception $e) {
               return '0';
           }*/
    }

    // protected function purchaseOrder($id, $request, $pay_type , $order_id)
    // {
    //     $process = \DB::transaction(function () use ($id, $request, $pay_type , $order_id) {
    //         $shop       = Badrshop::where('serial_id', $id)->firstOrFail();
    //         $bills_add_ = BillAdd::where('shop_id', $id)->get();
    //         $last_order = CartRequest::where('shop_id', $id)->select('order_no')->orderBy('id', 'DESC')->first();
    //         $last_order = is_null($last_order) ? 1 : $last_order->order_no + 1;
            
    //         if (in_array($order_id, [null, 0, '0'])) {
    //             $last_order = CartRequest::where('shop_id', $id)
    //                 ->select('order_no')
    //                 ->orderBy('id', 'DESC')
    //                 ->first();
    //             if (is_null($last_order)) {
    //                 $last_order = 1;
    //             } else {
    //                 $last_order = $last_order->order_no + 1;
    //             }
    //         } else {
    //             $last_order = CartRequest::where('shop_id', $id)
    //                 ->where('id', $order_id)
    //                 ->select('order_no')
    //                 ->first();

    //             $last_order = $last_order->order_no;
    //         }
            
    //         $settings              = RequestSettings::where('shop_id', $id)->first();
    //         $items                 = $request->order_details;
    //         $addres_lat            = $request->lat ?? 0;
    //         $addres_lon            = $request->lon ?? 0;
    //         $order_data            = [];
    //         $total                 = 0;
    //         $discount              = 0;
    //         $vat                   = 0;
    //         $net                   = 0;
    //         $all_fee               = 0;
    //         $not_cards             = [];
    //         $total_offer_discount  = 0;

    //         foreach ($items as $it) {
    //             $item_id    = $it['item_id'];
                
    //             $quantity   = $it['quantity'];
    //             $unit_id    = $it['unit_id'];
    //             $item_notes = $it['item_notes'] ?? null;
    //             $item       = Item::find($item_id);
    //             $isCard     = $this->cardCheck($item);

    //             if ($item->unit_id == $unit_id) {
    //                 $price = $item->sale_price;
    //             } else {
    //                 $unit  = ItemUnit::where('shop_id', $id)->where('unit_id', $unit_id)->where('item_id', $item_id)->first();
    //                 $price = $unit->unit_price;
    //             }

    //             $price_original = $price;
    //             $price_new      = $price;
    //             $discount_      = $item->withDiscount;
    //             $dis            = 0;

    //             if ($discount_) {
    //                 $price_new = $price_original - $this->calcItemDiscount($settings, $item, $price_original);
    //                 $dis       = $this->calcItemDiscount($settings, $item, $price_original) * $quantity;
    //             }

    //             $price_original = $price_new;

    //             if ($shop->bill_adds == 0) {
    //                 if ($item->vat_state == 2) {
    //                     $v = \DB::table('bills_add')
    //                         ->where('shop_id', $id)
    //                         ->where('id', $item->vat_id)->first();
    //                     if ($v) {
    //                         $price_original = $price_original;
    //                         if ($v->check_addition == 07) {
    //                             $price_original = $price_original / (1 + $v->addition_value / 100);
    //                         }
    //                     }
    //                 }

    //                 $price_new             = $price_original;
    //                 $item_total_price      = $price_new * $quantity;
    //                 $item_total_price_main = $price * $quantity;
    //                 $offer_discount        = $item_total_price_main - $item_total_price;
    //                 $total_offer_discount  += $offer_discount;
    //                 $discount             +=  $dis;
                     
    //                 // $discount              = $discount + $discount_ == 1 ? 0 : $dis;
    //                 // dd($discount);
    //                 $for_v                 = $item_total_price;
    //                 $total                += $item_total_price;
    //                 $item_vat              = 0;

    //                 if ($item->vat_state != '0') {
    //                     foreach ($bills_add_ as $add) {
    //                         if ($add->check_addition) {
    //                             $item_vat = $add->addition_value;
    //                             $vat += $item_vat;
    //                             $net += $for_v + $add->addition_value;
    //                         } else {
    //                             $item_vat = $for_v * $add->addition_value / 100;
    //                             $vat += $item_vat;
    //                             $net += $for_v + ($for_v * $add->addition_value / 100);
    //                         }
    //                     }
    //                 } else {
    //                     $net += $for_v;
    //                 }
    //             }else{
                    
    //                 $price_new             = $price_original;
    //                 $item_total_price      = $price_new * $quantity;
    //                 $discount              += $dis;
                   
    //                 //  $discount              = $discount + $discount_ == 1 ? 0 : $dis;
    //                 $total                += $item_total_price;
    //                 $item_vat              = 0;
    //                 $total_offer_discount += 0;
    //             }

    //             $im_fee       = 0;
                
    //             $order_data[] = [
    //                 'item_id'        => $item_id,
    //                 'unit_id'        => $unit_id,
    //                 'item_notes'     => $item_notes,
    //                 'quantity'       => $quantity,
    //                 'o_price'        => $price_original,
    //                 'price'          => $price_new,
    //                 'offer_discount' => ($price - $price_new) * $quantity,
    //                 'is_card'        => $isCard,

    //                 'vat'            => $item_vat,
    //                 'total'          => $item_total_price,
    //                 'fee'            => $request->fee
    //             ];
               
    //         }
            

    //         $net             = $total + $vat;
    //         $coupon_discount = 0;

    //         // if (in_array($id, [29760, '29760'])) { // For Demo Shop
    //             if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0', 0])) {
                  
    //                 $coupon    = Coupon::where('id', $request->coupon_id)->first();
    //                 $couponUse = $coupon->checkCouponUse(0);
    //                 if ($coupon && $coupon->check() && ($couponUse && $couponUse !== 'unAuthorized')) {
    //                     if ($couponUse !== 'feeDiscount') {
    //                         $coupon_discount = ($coupon->discount_type == 0) ? $coupon->discount_value * ($net + $all_fee) / 100 : $coupon->discount_value;
    //                     } else {
    //                         $coupon_discount = (($request->is_shipping == true || $settings->apply_fee == 1) &&
    //                                             (price_decimal($net, $id) <= price_decimal($settings->max_charge, $id)))
    //                                             ? $this->CalcFee($request, $net)
    //                                             : 0;
    //                     }
    //                 }
    //             }
    //         // } else { // For Live Shop
    //         //     if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0', 0])) {
    //         //         $coupon = Coupon::where('id', $request->coupon_id)->first();
    //         //         if ($coupon && $coupon->check()) {
    //         //             $coupon_discount = ($coupon->discount_type == 0) ? $coupon->discount_value * ($net + $all_fee) / 100 : $coupon->discount_value;
    //         //         }
    //         //     }
    //         // }

    //         $finalTotal = $net - $coupon_discount;
            
    //         // if (
    //         //     $settings->apply_fee && 
    //         //     $request->is_shipping &&
    //         //     price_decimal($finalTotal + $coupon_discount, $id) <= price_decimal($settings->max_charge, $id)
    //         // ) {
    //         //     $all_fee = $this->CalcFee($request, $net);
    //         // } else {
    //         //     $all_fee = 0;
    //         // }
            
    //         // if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0', 0])) {
    //         //     $coupon = Coupon::where('id', $request->coupon_id)->first();
    //         //     if ($coupon && $coupon->check()) {
    //         //         $coupon_discount = ($coupon->discount_type == 0) ? $coupon->discount_value * ($net + $all_fee) / 100 : $coupon->discount_value;
    //         //     }
    //         // }
            
    //         // $finalTotal = $net - $coupon_discount;
    //         // $all_fee    = (($request->is_shipping == true || $settings->apply_fee == 1) && (price_decimal($finalTotal, $id) <= price_decimal($settings->max_charge, $id))) ? $this->CalcFee($request, $net) : 0;
    //         // // $all_fee = (($request->is_shipping == true) && (price_decimal($net,$id) > price_decimal($settings->max_charge,$id))) ? $this->CalcFee($request, $net) : 0;

    //         $all_fee = 0;
    //         if($request->is_shipping ==false)
    //         {
    //             $all_fee = 0;
    //         }
    //         else
    //         {
    //              $all_fee = $this->CalcFee($request, $net);
    //         }

    //         if ($request->has('fast_charge') && !in_array($request->fast_charge, ['', null]) && $request->fast_charge == '1' && $settings) {
    //             if ($settings->fee_type == 0) {
    //                 $all_fee = $settings->fee / 100 * $net;
    //             }
    //         }
            
    //         if($request->is_shipping ==false)
    //         {
    //             $all_fee = 0;
    //         }

    //         $status = 1;
    //         if ($pay_type == 1) {
    //             $status = 4;
    //         }

    //         // $coupon_discount = 0;
    //         // if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0', 0])) {
    //         //     $coupon = Coupon::where('id', $request->coupon_id)->first();
    //         //     if ($coupon && $coupon->check()) {
    //         //         $coupon_discount = ($coupon->discount_type == 0) ? $coupon->discount_value * ($net + $all_fee) / 100 : $coupon->discount_value;
    //         //     }
    //         // }

    //         $c_id = $request->client_id ?? 0;
    //         if ($request->header('souq-albadr')) {
    //             $c_id = auth()->guard('client')->id();
    //         }

    //         // $discount              += $coupon_discount;
           
    //         $totalPrice             = $net + $all_fee - $coupon_discount;
    //         $requestc               = (in_array($order_id, [null, 0, '0'])) ? new CartRequest() : CartRequest::find($order_id);
    //         $requestc->client_id    = $c_id;
    //         $requestc->shop_id      = $id;
    //         $requestc->status       = $status;
    //         $requestc->lat          = $addres_lat;
    //         $requestc->lon          = $addres_lon;
    //         $requestc->total        = $total;
    //         $requestc->net          = $totalPrice;
    //         $requestc->fee          = $all_fee;
    //         $requestc->order_no     = $last_order;
    //         $requestc->fee_type     = $settings->fee_type;
    //         $requestc->client_name  = $request->client_name;
    //         $requestc->email        = $request->email;
    //         $requestc->mobile       = $request->mobile;
    //         $requestc->phone        = $request->phone;
    //         $requestc->payment_type = $pay_type;
    //         $requestc->city_id      = $request->city_id ?? null;
    //         $requestc->coupon_id    = $request->coupon_id ?? 0;
    //         $requestc->discount     = $discount;
    //         $requestc->coupon_discount     = $coupon_discount;
    //         $requestc->address      = $request->address;
    //         $requestc->is_shipping  = $request->is_shipping == true ? 1 : 0;
    //         $requestc->created_at   = date('Y-m-d H:i:s');
    //         $requestc->updated_at   = date('Y-m-d H:i:s');
    //         $requestc->save();
    //         $client =Client::where('id',$request->client_id)->first();
           
    //         if($client)
    //         {
    //             $client->update([
    //                 'mobile1' => $request->client_name ]);
                   
    //         }
    //         if ($order_id > 0) {
    //             $current_details = RequestDetails::where('request_id', $order_id)->get();
    //             foreach ($current_details as $cd) {
    //                 $cd->delete();
    //             }
    //         }

    //         $order_total = 0;
    //         foreach ($order_data as $data) {
    //             $it        = Item::find($data['item_id']);
    //             $cardCheck = $data['is_card'];

    //             if ($cardCheck != '0') {
    //                 $cards = $this->getCards($it, $data['quantity']);
    //                 foreach ($cards as $card) {
    //                     $card->request_id = $requestc->id;
    //                     $card->card_state = 1;
    //                     $card->save();
    //                 }
    //             }

    //             $details                 = new RequestDetails();
    //             $details->request_id     = $requestc->id;
    //             $details->item_id        = $data['item_id'];
    //             $details->unit_id        = $data['unit_id'];
    //             $details->quantity       = $data['quantity'];
    //             $details->item_notes     = $data['item_notes'];
    //             $details->price          = $data['price'];
    //             $details->shop_id        = $id;
    //             $details->offer_discount = $data['offer_discount'];
    //             // حفظ السعر الاساسي عند  حفظ الطلب لعرضه في الفاتوره 
    //             $details->sale_price = $it->sale_price;
    //             $details->pay_price = $it->pay_price;
    //             $details->discount_percent = $it->discount_percent ?? 0;
    //             $details->save();
    //             if ($it->vat_state != 0)
    //                 $order_total += $data['price'] * $details->quantity;
    //         }

    //         /*
    //         if (in_array($order_id, [null, 0, '0'])) {
    //             if (!in_array($it->vat_state, [0, '0'])) {
    //                 foreach ($bills_add_ as $iadd) {
    //                     // Added For AboSamra (If vat type is on purchase only)
    //                     if(in_array($iadd->add_role , [2 ,'2']) && !in_array($iadd->add_role , [3 ,'3'])){
    //                         $vati                              = $iadd->check_addition ? $iadd->addition_value : $iadd->addition_value / 100 * $order_total;
    //                         $bills_add_history                 = new BillAddHistory();
    //                         $bills_add_history->bill_id        = 0;
    //                         $bills_add_history->addition_id    = $iadd->id;
    //                         $bills_add_history->addition_value = $vati;
    //                         $bills_add_history->shop_id        = $id;
    //                         $bills_add_history->type           = 1;
    //                         $bills_add_history->request_id     = $requestc->id;
    //                         $bills_add_history->save();
    //                     }
    //                 }
    //             }
    //         } else {
    //             $bills_add_histories = BillAddHistory::where('request_id', $requestc->id)->where('shop_id', $id)->get();
    //             if(count($bills_add_histories) > 0){
    //                 foreach ($bills_add_histories as $index => $bills_add_history) {
    //                     $vati                              = $bills_add_[$index]->addition_value / 100 * $order_total;
    //                     $bills_add_history->bill_id        = 0;
    //                     $bills_add_history->addition_id    = $bills_add_[$index]->id;
    //                     $bills_add_history->addition_value = $vati;
    //                     $bills_add_history->shop_id        = $id;
    //                     $bills_add_history->type           = 1;
    //                     $bills_add_history->request_id     = $requestc->id;
    //                     $bills_add_history->save();
    //                 }
    //             }
    //         }
    //         */
    //         return [$requestc->order_no, $requestc->id, $requestc->net];
    //     });

    //     return $process;
    // }


    public function saveRequestOld(Request $request, $id)
    {
        
        $settings = RequestSettings::where('shop_id', $id)->first();
        if ($settings->dis_req_quantity == 1) {
            $avilable = $this->checkAvailability($request->order_details, $id);
            
            info('sdasdasdas', [$request->all(), $avilable]);
            
            if (!$avilable) {
                
                // return response()->json([
                //     'status' => false,
                //     'msg' => trans('theme.order_fail') . ' لعدم توافر الكمية '
                // ]);
                
                 return response()->json(['status' => false, 'msg' => 'لا يمكن حفظ الطلب لعدم توافر الكمية']);
            }
        }

        $type = $request->payment_type;


        $order = $this->purchaseOrder($id, $request, $type);
        if ($order != '0') {

            $infouser = Client::where('id', $request->client_id)->first();
            //            $infouser->city_id = $request->city_id;
            //            $infouser->save();
            //            dd($infouser);


            $content = 'تم تسجيل طلبك رقم  ' . $order[0] . ' بنجاح';
            $player_id = $infouser->player_id;
            $tokens = [$player_id];

            $headings = [
                'en' => 'بيت السبح',
                'ar' => 'bayt alsebah',
            ];
            $data = ['order_id' => $order, 'type' => 1];
            $client_notification = new ClientNotification();
            $client_notification->client_id = $infouser->id;
            $client_notification->order_id = $order[0];
            $client_notification->message = $content;
            $client_notification->type = 1;
            $client_notification->shop_id = $id;
            $client_notification->save();
            //            dd($client_notification);
            // if (!is_null($player_id)) {

            //     $x = sendNote($tokens, $content, $headings, $data);


            // }

            $msg = trans('theme.order_success') . " - " . trans('theme.order_no') . ":" . $order[0];

            $response = [
                'status' => true,
                "msg" => $msg,
                'order_id' => $order[1],
                'final_net' => price_decimal($order[2], $id)
            ];
            return response()->json($response, 201);
        } else {
            return response()->json([
                'status' => false,
                'msg' => trans('theme.order_fail')
            ]);
        }
        //flash()->success('تم تسجيل طلبك بنجاح');
        //        return $id;
    }

    protected function purchaseOrderOld($id, $request, $pay_type)
    {


        $process = \DB::transaction(function () use ($id, $request, $pay_type) {

            $shop = Badrshop::where('serial_id', $id)->firstOrFail();
            $bills_add_ = BillAdd::where('shop_id', $id)->get();

            $last_order = CartRequest::where('shop_id', $id)->select('order_no')
                ->orderBy('id', 'DESC')->first();
            if (is_null($last_order)) {
                $last_order = 1;
            } else {
                $last_order = $last_order->order_no + 1;
            }


            $settings = RequestSettings::where('shop_id', $id)->first();
          
            # الحد الاقصي لفرض الرسوم
            /*
            $fee = $settings->fee;
            $fee_type = $settings->fee_type;
            $max_charge = $settings->max_charge;
            */
            $items = $request->order_details;

            $addres_lat = $request->lat ?? 0;
            $addres_lon = $request->lon ?? 0;
            $order_data = [];
            $total = 0;
            $discount = 0;
            $vat = 0;
            $net = 0;
            $all_fee = 0;


            $not_cards = [];
            foreach ($items as $it) {
                $item_id = $it['item_id'];
                $quantity = $it['quantity'];
                $unit_id = $it['unit_id'];
                $item_notes = $it['item_notes'] ?? null;
               
                $item = Item::find($item_id);
 
                $isCard = $this->cardCheck($item);
                if ($item->unit_id == $unit_id) {
                    $price = $item->sale_price;
                } else {
                    $unit = ItemUnit::where('shop_id', $id)->where('unit_id', $unit_id)->where('item_id', $item_id)->first();
                    $price = $unit->unit_price;
                }

                $price_original = $price;
                $price_new = $price;
                $discount_ = $item->withDiscount;
                $dis = 0;
                if ($discount_) {
                    $dis = $item->discount_percent / 100 * $price_original * $quantity;
                    $price_new = (1 - ($item->discount_percent / 100)) * $price_original;
                    /* if ($item->vat_state == 2) {
                         $price_new = (1 - ($item->discount_percent / 100)) * $price;
                     }*/
                }
                $price_original = $price_new;

                if ($item->vat_state == 2) {
                    $v = \DB::table('bills_add')
                        ->where('shop_id', $id)
                        ->where('id', $item->vat_id)->first();


                    if ($v) {
                        $price_original = $price_original;
                        if ($v->check_addition == 0) {
                            $price_original = $price_original / (1 + $v->addition_value / 100);
                        }
                        //                        $price_original = price_decimal($price_original, $id);
                    }
                }
                $price_new = $price_original;

                $item_total_price = $price_new * $quantity;
                $item_total_price_main = $price * $quantity;

                //                $total_net = price_decimal(($item->sale_price) * $quantity, $id);
                $discount = $discount + $dis;

                $for_v = $item_total_price;
                $total += $item_total_price;
                $item_vat = 0;

                if ($item->vat_state != '0') {
                    foreach ($bills_add_ as $add) {
                        if ($add->check_addition) {
                            $item_vat = $add->addition_value;
                            $vat += $item_vat;
                            $net += $for_v + $add->addition_value;
                        } else {
                            $item_vat = $for_v * $add->addition_value / 100;
                            $vat += $item_vat;
                            $net += $for_v + ($for_v * $add->addition_value / 100);
                        }
                    }
                } else {
                    $net += $for_v;
                }
                $im_fee = 0;

                //                if ($isCard == '0') {
                //                    $not_cards[] = '1';
                /* $im_fee = $settings->fee;
                 if ($settings->fee_type == '0') {
                     $im_fee = $im_fee / 100 * ($item_total_price_main - $discount);
                 }*/
                //                }
                //                $all_fee += $im_fee;

                $order_data[] = [
                    'item_id' => $item_id,
                    'unit_id' => $unit_id,
                    'item_notes' => $item_notes,
                    'sale_price' =>$item->sale_price,
                    'quantity' => $quantity,
                    'o_price' => $price_original,
                    'price' => $price_new,
                    'is_card' => $isCard,
                    'discount' => $dis,
                    'vat' => $item_vat,
                    'total' => $item_total_price,
                    'fee' => $request->fee
                ];
            }
            if($shop->bill_adds == 1)
            {
                 $net = $total ; 
            }
            else
            {
                 $net = $total + $vat;
            }
           
            $all_fee = $this->CalcFee($request, $net);
            /* if(sizeof($not_cards) > 0){
                 if ($settings->fee_type == 0) {
                     $all_fee = $net * $settings->fee / 100;
                 } else {
                     $all_fee = $settings->fee;
                 }

                 if ($net > $settings->max_charge) {
                     $all_fee = 0;
                 }
             }*/

            //            $all_fee = $request->fee;

            //            $all_fee =$this->CalcFee($request);


            if ($request->has('fast_charge') && !in_array($request->fast_charge, ['', null]) && $request->fast_charge == '1' && $settings) {

                if ($settings->fee_type == 0) {
                    $all_fee = $settings->fee / 100 * $net;
                }
            }
            if($request->is_shipping ==false)
            {
                $all_fee = 0;
            }
            else
            {
                $all_fee =$all_fee;
            }
            $status = 1;
            if ($pay_type == 1) {
                $status = 4;
            }

            $coupon_discount = 0;
            $coupon_id =0;
            $client_order=CartRequest::where([
                'shop_id'=>$id,
                'client_id'=>$request->client_id,
            ])->first();
            if($client_order == null)
            {
                 if($settings->new_order_discount == 1)
                {
                    $coupon = Coupon::where('coupon_code', 'first_order_discount')->first();
                    if ($coupon && $coupon->check()) {
                        if ($coupon->discount_type == 0) {
                            $coupon_discount = $coupon->discount_value * ($total) / 100;
                        } else {
                            $coupon_discount = $coupon->discount_value;
                        }
                    }
                    $coupon_id =$coupon->id;
                }
            }
            else
            {
                if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0'])) {
                    $coupon = Coupon::where('id', $request->coupon_id)->first();
                    if ($coupon && $coupon->check()) {
                        if ($coupon->discount_type == 0) {
                            $coupon_discount = $coupon->discount_value * ($total) / 100;
                        } else {
                            $coupon_discount = $coupon->discount_value;
                        }
                    }
               }
               $coupon_id =$request->coupon_id;
            }
            
            

            // $discount += $coupon_discount;
            $requestc = new CartRequest();
            $requestc->client_id = $request->client_id ?? 0;
            $requestc->shop_id = $id;
            $requestc->status = $status;
            $requestc->lon = $addres_lat;
            $requestc->lat = $addres_lon;
            $requestc->total = $total;
            $requestc->net = $net + $all_fee - $coupon_discount;
            $requestc->fee = $all_fee;
            $requestc->order_no = $last_order;
            $requestc->fee_type = $settings->fee_type;
            $requestc->client_name = $request->client_name;
            $requestc->email = $request->email;
            $requestc->mobile = $request->mobile;
            $requestc->payment_type = $pay_type;
            $requestc->city_id = $request->city_id ?? null;
            $requestc->coupon_id = $coupon_id;
            $requestc->coupon_discount =$coupon_discount;

            /* if ($pay_type == 2) {
                 if ($request->hasFile('trans_file')) {

                     $file = $request->file('trans_file');
                     $ext = strtolower($file->getClientOriginalExtension());

                     $file_name = 'trans_' . date("Y-m-d") . '_' . time() . '.' . $ext;

                     $destinationPath = '../c-admin/upload/trans_files';

                     $file->move($destinationPath, $file_name);
                     $requestc->trans_img = $file_name;
                 }
             }*/

            $requestc->discount = $discount;

            $requestc->address = $request->address;

            $requestc->created_at = date('Y-m-d H:i:s');
            $requestc->updated_at = date('Y-m-d H:i:s');

            $requestc->save();

            $order_total = 0;

            foreach ($order_data as $data) {
                $it = Item::find($data['item_id']);

                //                if ($pay_type == '1') {
                $cardCheck = $data['is_card'];
                if ($cardCheck != '0') {
                    $cards = $this->getCards($it, $data['quantity']);
                    //dd( $data['quantity'], count($cards), $cards);
                    foreach ($cards as $card) {
                        $card->request_id = $requestc->id;
                        $card->card_state = 1;
                        $card->save();
                    }
                }
                //                }

                $details = new RequestDetails();
                $details->request_id = $requestc->id;
                $details->item_id = $data['item_id'];

                //                $details->size_id = $item->attributes['size'];
                //                $details->color_id = $item->attributes['color'];
                $details->unit_id = $data['unit_id'];
                $details->quantity = $data['quantity'];
                $details->item_notes = $data['item_notes'];
                $details->price = $data['price'];
              $details->sale_price = $it->sale_price;
                $details->shop_id = $id;
                //dd($details->size_id ,$details->color_id);
                $details->save();
                if ($it->vat_state != 0)
                    $order_total += $data['price'] * $details->quantity;
            }


            //            dd($order_total);
            foreach ($bills_add_ as $iadd) {
                if ($iadd->check_addition) {
                    $vati = $iadd->addition_value;
                } else {
                    $vati = $iadd->addition_value / 100 * $order_total;
                }
                $bills_add_history = new BillAddHistory();
                $bills_add_history->bill_id = 0;
                $bills_add_history->addition_id = $iadd->id;
                $bills_add_history->addition_value = $vati;
                $bills_add_history->shop_id = $id;
                $bills_add_history->type = 1;
                $bills_add_history->request_id = $requestc->id;
                $bills_add_history->save();
            }

            return [$requestc->order_no, $requestc->id, $requestc->net];

            //            return $data;

        });
        return $process;
        /*   } catch (\Exception $e) {
               return '0';
           }*/
    }

    protected function cardCheck($item)
    {
        $cardCompany_id = $item->card_company_id;
        if (in_array($cardCompany_id, ['0', '', null])) {
            return '0';
        }
        return $cardCompany_id;
    }

    protected function getCards($item, $qty)
    {
        $cardCheck = $this->cardCheck($item);

        if ($cardCheck != '0') {
            $cards_query = Card::where([
                'shop_id' => $item->shop_id,
                'item_id' => $item->id,
                'request_id' => 0,
                'sale_id' => 0,
                'card_state' => 0
            ]);
            $cards = $cards_query->take($qty)->get();

            return $cards;
        } else {
            return false;
        }
    }

    public function saveRequesImg(Request $request)
    {
        $shop_id = $request->shop_id;
        $order_id = $request->order_id;
        $requestc = CartRequest::where([
            'shop_id' => $shop_id,
            'id' => $order_id
        ])->first();
        if ($requestc) {
            if ($request->hasFile('trans_file')) {
                $file = $request->file('trans_file');
                $ext = strtolower($file->getClientOriginalExtension());
                $file_name = 'trans_' . date("Y-m-d") . '_' . time() . '.' . $ext;
                $destinationPath = '../c-admin/upload/trans_files';
                $file->move($destinationPath, $file_name);
                $requestc->trans_img = $file_name;
            }
            $requestc->save();
        }
        return response()->json([
            'status' => true,
            'msg' => trans('theme.save_success')
        ]);
    }

    public function checkCoupon($shop_id, Request $request)
    {
        $serial = $request->coupon;
        $coupon = Coupon::where('shop_id', $shop_id)
            ->where('coupon_code', $serial)
            ->first();

        if ($coupon) {
            if ($coupon->check() && $coupon->checkCouponUse(0)) {
                /*
                                $total = $request->total;
                                $discount = $coupon->discount_value;
                                $discount_type = $coupon->discount_type;

                                if ($discount_type == 0) {
                                    $x = $discount * $total / 100;
                                    $n_total = $total - $x;
                                } else {
                                    $n_total = $total - $discount;
                                }

                                return response()->json([
                                    'status' => true,
                                    'message' => 'تم تفعيل الكوبون بنجاح',
                                    'n_total' => $n_total
                                ]);*/

                return response()->json([
                    'status' => true,
                    'message' => trans('theme.coupon_success'),
                    'coupon_id' => $coupon->id,
                    'value' => $coupon->discount_value,
                    'type' => $coupon->discount_type
                ]);
            } else {
                return response()->json([
                    'status' => false,
                    'message' => $coupon->getStandString()
                ]);
            }
        } else {
            return response()->json([
                'status' => false,
                'message' => trans('theme.no_coupon')
            ]);
        }
    }

    // public function getDistancePrice(Request $request)
    // {
    //     $distance = $request->distance;

    //     $price = ShippingDistancesPrice::where('shop_id', $request->shop_id)->whereRaw(" $distance  BETWEEN km_from AND km_to")->first();

    //     if ($price) {
    //         $fee = price_decimal($distance * $price->price_per_kilo, $request->shop_id);
    //     } else {
    //         $fee = 0;
    //     }

    //     return response()->json([
    //         'status' => true,
    //         'distance_price' => $fee,

    //     ]);
    // }
    public function getDistancePrice(Request $request)
    {
        $fee = $this->CalcFee($request, $request->net);
        return response()->json(['status' => true, 'distance_price' => $fee]);
    }
}
