<?php

namespace App\Http\Controllers\ApiV2\Services;
use App\Badrshop;
use App\BillAdd;
use App\CartRequest;
use App\Coupon;
use App\Item;
use App\RequestDetails;
use App\RequestSettings;

class RequestService {

    const SAVE=0;
    const UPDATE=1;
    protected Badrshop $shop;
    protected $shop_id;
    protected $bill_adds;
    protected RequestSettings $request_settings;
    protected $order;
    protected $order_id;
    protected $order_no;
    protected $action_type = self::SAVE;
    protected $order_details;
    protected $request_details;
    protected $request;
    protected $coupon;

    public function __construct() {
    }

    public function setShop($shop_id) {
        $this->shop = Badrshop::where('serial_id', $shop_id)->firstOrFail();
        $this->shop_id = $shop_id;
        return $this;
    }

    public function setRequestSettings() {
        $this->request_settings = RequestSettings::where('shop_id', $this->shop_id)->first();
        return $this;
    }

    public function getRequestSettings() {
        return $this->request_settings;
    }

    public function setBillAdds() {
        $this->bill_adds = BillAdd::where('shop_id', $this->shop_id)->get();
        return $this;
    }
    
    public function getBillAdds() {
        return $this->bill_adds;
    }

    public function setOrder($order_id) {
        if (in_array($order_id, [null, 0, '0'])) {
            $this->order = null;
            $this->order_id = null;
            $this->action_type = self::SAVE;
        } else {
            $this->action_type = self::UPDATE;
            $this->order_id = $order_id;
            $this->order = CartRequest::where('shop_id', $this->shop_id)
                ->where('id', $order_id)
                ->first();
        }
        return $this;
    }

    public function getActionType() {
        return $this->action_type;
    }

    public function isUpdateRequest() {
        return $this->action_type == self::UPDATE;
    }

    public function isSaveRequest() {
        return $this->action_type == self::SAVE;
    }

    public function setOrderIndex() {
        $this->order_no = CartRequest::getNextOrderIndex(
            $this->action_type,
            $this->shop_id,
            $this->order_id);
        return $this;
    }

    public function getOrderIndex() {
        return $this->order_no;
    }

    public function setRequest($request) {
        $this->request = $request;
        $this->request->lat = $request->lat ?? 0;
        $this->request->lon = $request->lon ?? 0;
        $this->setCouponFromRequest();
        return $this;
    }

    public function setRequestDetails($request_details) {
        $this->request_details = $request_details ?? $this->request->order_details;
        return $this;
    }

    public function setOrderDetails() {
        $this->order_details = RequestDetails::where('request_id', $this->order_id)
            ->where('shop_id', $this->shop_id)
            ->get();
        return $this;
    }

    public function setCouponFromRequest() {
        $this->coupon = Coupon::where('id', $this->request->coupon_id)->first();
        return $this;
    }

    public function getOrderData() {
        $order_data = array();
        $this->checkItemDiscountQty();
        foreach ($this->request_details as $detail) {
            $item_id = $detail['item_id'];
            $quantity = $detail['quantity'];
            $unit_id = $detail['unit_id'];
            $item_notes = $detail['item_notes'] ?? null;
            $item = Item::find($item_id);

            $price = $item->itemPrice(
                $unit_id,
                $quantity,
                $this->action_type,
                $this->order_details->where('item_id', $item->id),
                $this->shop_id);

            $price_after_discount = $price;
            $discount_percent = 0;
            if (!isset($detail['withDiscount']) || $detail['withDiscount']) {
                $price_after_discount = $item->itemPriceAfterDiscount($price, $this->request_settings->discount_type);
                $discount_percent = $this->request_settings->discount_type == '' ? $item->discount_percent : (100 - (($price_after_discount / $price) * 100));
            }

            $price_after_vat = $item->itemPriceAfterVat($price_after_discount, $this->bill_adds);

            $all_fee = 0;
            $deliver_fee = $this->request_settings->fee;
            if ($this->request_settings->fee_type == '0') {
                $deliver_fee = $deliver_fee / 100 * $price;
            }
            $all_fee += $deliver_fee;

            $order_data[] = [
                'item_id' => $item_id,
                'unit_id' => $unit_id,
                'item_notes' => $item_notes,
                'quantity' => $quantity,
                'o_price' => $price,
                'price' => $price,
                'discount_percent' => $discount_percent,
                'price_after_discount' => $price_after_discount,
                'price_after_vat' => $price_after_vat,
                'discount' => $price - $price_after_discount,
                'vat' => $price_after_vat - $price_after_discount,
                'total_main' => $price,
                'total_after_discount' => $price_after_discount,
                'total_after_coupon' => $price_after_discount,
                'fee' => $all_fee
            ];
        }
        
        return $order_data;
    }
    
    protected function checkItemDiscountQty()
    {
        if (!$this->request_settings->enable_max_item_qty_discount) return ;
        $details = [];
        foreach ($this->request_details as $detail) {
            if ($item = Item::select('discount_percent', 'discount_quantity')->where('withDiscount', 1)->where('discount_percent', '>', 0)->find($detail['item_id'])) {
                if ($detail['quantity'] > $item->discount_quantity) {
                    $details[] = [
                        "item_id"      => $detail['item_id'],
                        "qty_problem"  => $detail['qty_problem'],
                        "quantity"     => $item->discount_quantity,
                        "unit_id"      => $detail['unit_id'],
                        "withDiscount" => true
                    ];
                        
                    $detail['withDiscount'] = false;
                    $detail['quantity']    -= $item->discount_quantity;
                    $details[] = $detail;
                } else {
                    $detail['withDiscount'] = true;
                    $details[] = $detail;
                }
            } else {
                $detail['withDiscount'] = false;
                $details[] = $detail;
            }
        }
        $this->request_details = $details;
    }
}
