<?php

namespace App\Http\Controllers\ApiV2;

use App\AppClient;
use App\Badrshop;
use App\BillAdd;
use App\BillAddHistory;
use App\Card;
use App\CartRequest;
use App\Client;
use App\ClientNotification;
use App\Coupon;
use App\Http\Controllers\Controller;
use App\Item;
use App\ItemUnit;
use App\RequestDetails;
use App\RequestSettings;
use App\ShippingDistancesPrice;
use Illuminate\Http\Request;
use App\Http\Controllers\ApiV2\BasicController;
use App\Http\Requests;
use Carbon\Carbon;

class NewOrderController extends BasicController
{
    public function __construct(Request $request)
    {
        app()->setLocale($request->header('Accept-Language') ?? 'ar');
    }

    private function checkAvailability($orderData, $shop_id, $order_id)
    {
        $available_items = [];
        $available_cards = [];

        if (is_array($orderData) && sizeof($orderData) > 0) {
            foreach ($orderData as $it) {
                $item_id = $it['item_id'];
                $quantity = $it['quantity'];
                $unit_id = $it['unit_id'];

                $item = Item::find($item_id);

                if ($item->unit_id != $unit_id) {
                    $unit = ItemUnit::where('shop_id', $shop_id)->where('unit_id', $unit_id)->where('item_id', $item_id)->first();
                    $quantity = ($unit->unit_value ?? 1) * $quantity;
                }

                $settings = $this->getOrderSettings($shop_id);
                $max_count = $settings->max_items;

                if (!in_array($item->card_company_id, ['', '0', null])) {
                    $max_count = $settings->max_cards;
                    $card_count = $this->cardCount($item);

                    if ($card_count < $max_count) {
                        $max_count = $card_count;
                    }
                } else {
                    if ($settings->dis_req_quantity == 1) {

                        $getRequests = \DB::table('request_details')
                            ->join('requests', function ($q) use ($item) {
                                $q->on('request_details.request_id', '=', 'requests.id');
                                $q->where('request_details.item_id', '=', $item->id);
                            })
                            ->leftJoin('items_unit', function ($q) {
                                $q->on('items_unit.id', '=', 'request_details.unit_id');
                                $q->where('items_unit.item_id', '=', 'request_details.item_id');
                            })
                            ->where('requests.no_bill', null)
                            ->whereNotIn('requests.status', ['0'])
                            ->selectRaw('if( items_unit.id > 0,  items_unit.unit_value * quantity , quantity ) as quantity')
                            ->sum('quantity');

                        $minus_qty_from_order = 0;

                        if ($order_id > 0) {

                            $detailsReduced  = RequestDetails::leftJoin('items_unit', function ($q) {
                                $q->on('items_unit.unit_id', '=', 'request_details.unit_id');
                                $q->on('items_unit.item_id', '=', 'request_details.item_id');
                            })
                                ->where('request_id', $order_id)
                                ->where('request_details.item_id', $item_id)
                                ->where('request_details.unit_id', $unit_id)
                                ->selectRaw('if( items_unit.id > 0, items_unit.unit_value * quantity ,quantity ) as quantity')
                                ->first();

                            $minus_qty_from_order = $detailsReduced->quantity ?? 0;
                        }
                        $getRequests = $getRequests - $minus_qty_from_order;
                    } else {
                        $getRequests = 0;
                    }

                    $newQty = $item->quantity - $getRequests;

                    if ($newQty <= 0) {
                        $max_count = 0;
                    }

                    if ($newQty < $max_count && $newQty > 0) {
                        $max_count = $newQty;
                    }
                }

                $item->quantity = (float)$max_count;

                if ($item && $item->quantity >= $quantity) {
                    $available_items[] = '1';
                    $is_com = $this->cardCheck($item);

                    if ($is_com != '0') {
                        $required_qty = $quantity;

                        $cards_query = Card::where([
                            'shop_id' => $item->shop_id,
                            'item_id' => $item_id,
                            'card_state' => 0,
                        ]);

                        $saved_qty = $cards_query->count();

                        if ($required_qty > $saved_qty) {

                            $neededQty = $required_qty - $saved_qty;

                            if ($neededQty > 0) {
                                $available_cards[] = '1';
                            }
                        }
                    }
                }
            }
        }

        if (sizeof($available_items) == sizeof($orderData) && sizeof($available_cards) == 0) {
            return true;
        }
        return false;
    }

    private function getOrderSettings($shop_id)
    {
        $settings = RequestSettings::where('shop_id', $shop_id)->first();
        if (is_null($settings)) {
            $settings = new RequestSettings();
            $settings->fee = 0;
            $settings->min_purchase = 0;
            $settings->max_charge = 0;
            $settings->max_cards = 5;
            $settings->max_items = 20;
            $settings->shop_id = $shop_id;
            $settings->save();
        }

        return $settings;
    }

    public function saveRequest(Request $request, $id)
    {
        $settings = RequestSettings::where('shop_id', $id)->first();
        $shop = Badrshop::where('serial_id', $id)->firstOrFail();
        $new_details = $request->order_details;

        if ($request->has('order_id') && $request->order_id > 0) {

            $order_id = $request->order_id;

            $currentorder = CartRequest::find($order_id);

            if ($currentorder->status != 1) {
                return response()->json(['status' => false, 'msg' => 'لا يمكن التعديل لان تم تأكيد الطلب من قبل الإدارة']);
            }
        }

        if ($settings->dis_req_quantity == 1) {
            $avilable = $this->checkAvailability($new_details, $id, $request->order_id ?? null);
            if (!$avilable) {
                return response()->json(['status' => false, 'msg' => 'لا يمكن حفظ الطلب لعدم توافر الكمية']);
            }
        }

        $type = $request->payment_type;

        $order = $this->purchaseOrder($id, $request, $type, $order_id ?? null);
        if ($order != '0') {

            $infouser  = Client::where('id', $request->client_id)->first();
            $content   = 'تم تسجيل طلبك رقم  ' . $order[0] . ' بنجاح';
            $player_id = $infouser->player_id;
            $tokens    = [$player_id];

            $headings = [
                'en' => $shop->shop_name,
                'ar' => $shop->shop_name,
            ];

            $data                           = ['order_id' => $order, 'type' => 1];
            $client_notification            = new ClientNotification();
            $client_notification->client_id = $infouser->id;
            $client_notification->order_id  = $order[0];
            $client_notification->message   = $content;
            $client_notification->type      = 1;
            $client_notification->shop_id   = $id;
            $client_notification->save();

            $msg = trans('theme.order_success') . " - " . trans('theme.order_no') . ":" . $order[0];

            $response = [
                'status'    => true,
                "msg"       => $msg,
                'order_id'  => $order[1],
                'final_net' => price_decimal($order[2], $id)
            ];
            return response()->json($response, 201);
        } else {
            return response()->json([
                'status' => false,
                'msg'    => trans('theme.order_fail')
            ]);
        }
    }

    protected function purchaseOrder($id, $request, $pay_type, $order_id)
    {
        $process = \DB::transaction(function () use ($id, $request, $pay_type, $order_id) {
            $shop       = Badrshop::where('serial_id', $id)->firstOrFail();
            $bills_add_ = BillAdd::where('shop_id', $id)->get();
            $last_order = CartRequest::where('shop_id', $id)->select('order_no')->orderBy('id', 'DESC')->first();
            $last_order = is_null($last_order) ? 1 : $last_order->order_no + 1;

            if (in_array($order_id, [null, 0, '0'])) {
                $last_order = CartRequest::where('shop_id', $id)
                    ->select('order_no')
                    ->orderBy('id', 'DESC')
                    ->first();
                if (is_null($last_order)) {
                    $last_order = 1;
                } else {
                    $last_order = $last_order->order_no + 1;
                }
            } else {
                $last_order = CartRequest::where('shop_id', $id)
                    ->where('id', $order_id)
                    ->select('order_no')
                    ->first();

                $last_order = $last_order->order_no;
            }

            $settings   = RequestSettings::where('shop_id', $id)->first();
            $items      = $request->order_details;
            $addres_lat = $request->lat ?? 0;
            $addres_lon = $request->lon ?? 0;
            $order_data = [];
            $total      = 0;
            $discount   = 0;
            $vat        = 0;
            $net        = 0;
            $all_fee    = 0;
            $not_cards  = [];

            foreach ($items as $it) {
                $item_id    = $it['item_id'];
                $quantity   = $it['quantity'];
                $unit_id    = $it['unit_id'];
                $item_notes = $it['item_notes'] ?? null;
                $item       = Item::find($item_id);
                $isCard     = $this->cardCheck($item);

                if ($item->unit_id == $unit_id) {
                    $price = $item->sale_price;
                } else {
                    $unit  = ItemUnit::where('shop_id', $id)->where('unit_id', $unit_id)->where('item_id', $item_id)->first();
                    $price = $unit->unit_price;
                }

                $price_original = $price;
                $price_new      = $price;
                $discount_      = $item->withDiscount;
                $dis            = 0;

                if ($discount_) {
                    $dis       = $item->discount_percent / 100 * $price_original * $quantity;
                    $price_new = (1 - ($item->discount_percent / 100)) * $price_original;
                }

                $price_original = $price_new;

                if ($shop->bill_adds == 0 || $settings->apply_vat == 1) {
                    if ($item->vat_state == 2) {
                        $v = \DB::table('bills_add')
                            ->where('shop_id', $id)
                            ->where('id', $item->vat_id)->first();
                        if ($v) {
                            $price_original = $price_original;
                            if ($v->check_addition == 0) {
                                $price_original = $price_original / (1 + $v->addition_value / 100);
                            }
                        }
                    }

                    $price_new             = $price_original;
                    $item_total_price      = $price_new * $quantity;
                    $item_total_price_main = $price * $quantity;
                    $discount              = $discount + $discount_ == 1 ? 0 : $dis;
                    $for_v                 = $item_total_price;
                    $total                += $item_total_price;
                    $item_vat              = 0;

                    if ($item->vat_state != '0') {
                        foreach ($bills_add_ as $add) {
                            if ($add->check_addition) {
                                $item_vat = $add->addition_value;
                                $vat += $item_vat;
                                $net += $for_v + $add->addition_value;
                            } else {
                                $item_vat = $for_v * $add->addition_value / 100;
                                $vat += $item_vat;
                                $net += $for_v + ($for_v * $add->addition_value / 100);
                            }
                        }
                    } else {
                        $net += $for_v;
                    }
                } else {
                    $price_new             = $price_original;
                    $item_total_price      = $price_new * $quantity;
                    $discount              = $discount + $dis;
                    $total                += $item_total_price;
                    $item_vat              = 0;
                }

                $im_fee       = 0;
                $order_data[] = [
                    'item_id'    => $item_id,
                    'unit_id'    => $unit_id,
                    'item_notes' => $item_notes,
                    'quantity'   => $quantity,
                    'o_price'    => $price_original,
                    'price'      => $price_new,
                    'is_card'    => $isCard,
                    'discount'   => $discount,
                    'vat'        => $item_vat,
                    'total'      => $item_total_price,
                    'fee'        => $request->fee
                ];
                
            }

            $net     = $total + $vat;
            $all_fee = (($request->is_shipping == true || $settings->apply_fee == 1) && (price_decimal($net, $id) > price_decimal($settings->max_charge, $id))) ? $this->CalcFee($request, $net) : 0;

            if ($request->has('fast_charge') && !in_array($request->fast_charge, ['', null]) && $request->fast_charge == '1' && $settings) {
                if ($settings->fee_type == 0) {
                    $all_fee = $settings->fee / 100 * $net;
                }
            }

            $status = 1;
            if ($pay_type == 1) {
                $status = 4;
            }

            $coupon_discount = 0;
            if ($request->has('coupon_id') && !in_array($request->coupon_id, ['', null, '0', 0])) {
                $coupon = Coupon::where('id', $request->coupon_id)->first();
                if ($coupon && $coupon->check()) {
                    $coupon_discount = ($coupon->discount_type == 0) ? $coupon->discount_value * ($net + $all_fee) / 100 : $coupon->discount_value;
                }
            }

            $c_id = $request->client_id ?? 0;
            if ($request->header('souq-albadr')) {
                $c_id = auth()->guard('client')->id();
            }

            $discount              += $coupon_discount;
            $totalPrice             = $net + $all_fee - $coupon_discount;
            $requestc               = (in_array($order_id, [null, 0, '0'])) ? new CartRequest() : CartRequest::find($order_id);
            $requestc->client_id    = $c_id;
            $requestc->shop_id      = $id;
            $requestc->status       = $status;
            $requestc->lat          = $addres_lat;
            $requestc->lon          = $addres_lon;
            $requestc->total        = $total;
            $requestc->net          = $totalPrice;
            $requestc->fee          = $all_fee;
            $requestc->order_no     = $last_order;
            $requestc->fee_type     = $settings->fee_type;
            $requestc->client_name  = $request->client_name;
            $requestc->email        = $request->email;
            $requestc->mobile       = $request->mobile;
            $requestc->payment_type = $pay_type;
            $requestc->city_id      = $request->city_id ?? null;
            $requestc->coupon_id    = $request->coupon_id ?? 0;
            $requestc->discount     = $discount;
            $requestc->address      = $request->address;
            $requestc->created_at   = date('Y-m-d H:i:s');
            $requestc->updated_at   = date('Y-m-d H:i:s');
            $requestc->save();

            if ($order_id > 0) {
                $current_details = RequestDetails::where('request_id', $order_id)->get();
                foreach ($current_details as $cd) {
                    $cd->delete();
                }
            }

            $order_total = 0;
            foreach ($order_data as $data) {
                $it        = Item::find($data['item_id']);
                $cardCheck = $data['is_card'];

                if ($cardCheck != '0') {
                    $cards = $this->getCards($it, $data['quantity']);
                    foreach ($cards as $card) {
                        $card->request_id = $requestc->id;
                        $card->card_state = 1;
                        $card->save();
                    }
                }

                $details             = new RequestDetails();
                $details->request_id = $requestc->id;
                $details->item_id    = $data['item_id'];
                $details->unit_id    = $data['unit_id'];
                $details->quantity   = $data['quantity'];
                $details->item_notes = $data['item_notes'];
                $details->price      = $data['price'];
                $details->shop_id    = $id;
                $details->save();
                if ($it->vat_state != 0)
                    $order_total += $data['price'] * $details->quantity;
            }

            if ($settings->apply_vat == 1) {
                if (in_array($order_id, [null, 0, '0'])) {
                    if (!in_array($it->vat_state, [0, '0'])) {
                        foreach ($bills_add_ as $iadd) {
                            $vati                              = $iadd->check_addition ? $iadd->addition_value : $iadd->addition_value / 100 * $order_total;
                            $bills_add_history                 = new BillAddHistory();
                            $bills_add_history->bill_id        = 0;
                            $bills_add_history->addition_id    = $iadd->id;
                            $bills_add_history->addition_value = $vati;
                            $bills_add_history->shop_id        = $id;
                            $bills_add_history->type           = 1;
                            $bills_add_history->request_id     = $requestc->id;
                            $bills_add_history->save();
                        }
                    }
                } else {
                    $bills_add_histories = BillAddHistory::where('request_id', $requestc->id)->where('shop_id', $id)->get();
                    if (count($bills_add_histories) > 0) {
                        foreach ($bills_add_histories as $index => $bills_add_history) {
                            $vati                              = $bills_add_[$index]->addition_value / 100 * $order_total;
                            $bills_add_history->bill_id        = 0;
                            $bills_add_history->addition_id    = $bills_add_[$index]->id;
                            $bills_add_history->addition_value = $vati;
                            $bills_add_history->shop_id        = $id;
                            $bills_add_history->type           = 1;
                            $bills_add_history->request_id     = $requestc->id;
                            $bills_add_history->save();
                        }
                    }
                }
            }

            return [$requestc->order_no, $requestc->id, $requestc->net];
        });

        return $process;
    }

    protected function cardCheck($item)
    {
        $cardCompany_id = $item->card_company_id;
        if (in_array($cardCompany_id, ['0', '', null])) {
            return '0';
        }
        return $cardCompany_id;
    }

    protected function getCards($item, $qty)
    {
        $cardCheck = $this->cardCheck($item);

        if ($cardCheck != '0') {
            $cards_query = Card::where([
                'shop_id' => $item->shop_id,
                'item_id' => $item->id,
                'request_id' => 0,
                'sale_id' => 0,
                'card_state' => 0
            ]);
            $cards = $cards_query->take($qty)->get();

            return $cards;
        } else {
            return false;
        }
    }

    public function saveRequesImg(Request $request)
    {
        $shop_id = $request->shop_id;
        $order_id = $request->order_id;
        $requestc = CartRequest::where([
            'shop_id' => $shop_id,
            'id' => $order_id
        ])->first();
        if ($requestc) {
            if ($request->hasFile('trans_file')) {
                $file = $request->file('trans_file');
                $ext = strtolower($file->getClientOriginalExtension());
                $file_name = 'trans_' . date("Y-m-d") . '_' . time() . '.' . $ext;
                $destinationPath = '../c-admin/upload/trans_files';
                $file->move($destinationPath, $file_name);
                $requestc->trans_img = $file_name;
            }
            $requestc->save();
        }
        return response()->json([
            'status' => true,
            'msg' => trans('theme.save_success')
        ]);
    }

    public function checkCoupon($shop_id, Request $request)
    {
        $serial = $request->coupon;
        $coupon = Coupon::where('shop_id', $shop_id)
            ->where('coupon_code', $serial)
            ->first();

        if ($coupon) {
            if ($coupon->check()) {
                /*
                                $total = $request->total;
                                $discount = $coupon->discount_value;
                                $discount_type = $coupon->discount_type;

                                if ($discount_type == 0) {
                                    $x = $discount * $total / 100;
                                    $n_total = $total - $x;
                                } else {
                                    $n_total = $total - $discount;
                                }

                                return response()->json([
                                    'status' => true,
                                    'message' => 'تم تفعيل الكوبون بنجاح',
                                    'n_total' => $n_total
                                ]);*/

                return response()->json([
                    'status' => true,
                    'message' => trans('theme.coupon_success'),
                    'coupon_id' => $coupon->id,
                    'value' => $coupon->discount_value,
                    'type' => $coupon->discount_type
                ]);
            } else {
                return response()->json([
                    'status' => false,
                    'message' => $coupon->getStandString()
                ]);
            }
        } else {
            return response()->json([
                'status' => false,
                'message' => trans('theme.no_coupon')
            ]);
        }
    }

    // public function getDistancePrice(Request $request)
    // {
    //     $distance = $request->distance;
    //     $price    = ShippingDistancesPrice::where('shop_id', $request->shop_id)->whereRaw(" $distance  BETWEEN km_from AND km_to")->first();
    //     $fee      = $price ? price_decimal($distance * $price->price_per_kilo, $request->shop_id) : 0;

    //     return response()->json(['status' => true,'distance_price' => $fee]);
    // }

    public function getDistancePrice(Request $request)
    {
        $fee = $this->CalcFee($request, $request->net);
        return response()->json(['status' => true, 'distance_price' => $fee]);
    }
}
