<?php

namespace App\Http\Controllers\ApiV4;

use Validator;
use App\Item;
use App\Client;
use Carbon\Carbon;
use App\WaitingList;
use Illuminate\Http\Request;

class WaitingListController extends CategoriesController
{
    public function index(Request $request)
    {
        $client       = auth()->guard('client')->user();
        $online_units = $this->getAvailableUnits($request->shop_id);
        $waitingList  = WaitingList::where('client_id', $client->id)->where('shop_id', $request->shop_id)->get();

        if ($waitingList->count() > 0 && $client) {
            $data = [];
            foreach ($waitingList as $waiting) {
                $item = Item::whereIn('sale_unit', $online_units)
                    ->where('online', '1')
                    ->where('available', '1')
                    ->where('shop_id', $request->shop_id)
                    ->find($waiting->product_id);

                if ($item) {
                    $data[] = [
                        'item_id'         => $waiting->product_id,
                        'item_name'       => $item->item_name,
                        'item_category'   => $item->category->name,
                        'client_id'       => $waiting->client_id,
                        'image'           => $item->getImg($request->shop_id),
                        'client_name'     => $client->client_name,
                        'card_company_id' =>  $item->card_company_id,
                        'price'           => price_decimal($item->sale_price, $request->shop_id)
                    ];
                }
            }

            return response()->json(['status' => true, 'data' => $data], 200);
        }

        return response()->json(['status' => false, 'msg' => 'لا يوجد منتجات بقائمة الأنتظار'], 200);
    }

    public function store(Request $request)
    {
        try {
            $rules = [
                'shop_id'    => 'required',
                'product_id' => 'required'
            ];

            $messages = [
                'shop_id.required'    => 'كود الدعم مطلوب',
                'product_id.required' => 'الصنف مطلوب'
            ];

            $validator = Validator::make($request->all(), $rules, $messages);

            if ($validator->fails()) {
                return response()->json(['status' => false, 'msg' => $validator->errors()->all()], 422);
            }

            // Check if product is already in waiting list
            $waitingList = WaitingList::where('client_id', auth('client')->id())
                ->where('product_id', $request->product_id)
                ->where('shop_id', $request->shop_id)
                ->first();
                
            if (!$waitingList) {
                $waiting = WaitingList::create([
                    'shop_id'     => $request->shop_id,
                    'product_id'  => $request->product_id,
                    'client_id'   => auth('client')->id(),
                    'is_notified' => 0,
                ]);

                if ($waiting) {
                    $response = [
                        'status' => true,
                        'msg'    => 'تم الإضافة لقائمة الأنتظار',
                        'data'   => [
                            'item_id'       => $waiting->product_id,
                            'item_name'     => $waiting->product->item_name,
                            'item_category' => $waiting->product->category->name,
                            'client_id'     => $waiting->client_id,
                            'client_name'   => $waiting->client != null ? $waiting->client->client_name : 'null',
                            'price'         => price_decimal($waiting->product->sale_price, $request->shop_id),
                            'created_at'    => $waiting->created_at,
                        ]
                    ];

                    return response()->json($response, 200);
                }
            }

            return response()->json(['status' => false, 'msg' => 'هذا المنتج موجود بالفعل في قائمة الأنتظار'], 422);
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'msg' => $e->getMessage()], 422);
        }
    }

    public function destroy(Request $request)
    {
        try{
            $rules = [
                'shop_id'    => 'required',
                'product_id' => 'required'
            ];

            $messages = [
                'shop_id.required'    => 'كود الدعم مطلوب',
                'product_id.required' => 'الصنف مطلوب'
            ];

            $validator = Validator::make($request->all(), $rules, $messages);

            if ($validator->fails()) {
                return response()->json(['status' => false, 'msg' => $validator->errors()->all()], 422);
            }

            $waiting = WaitingList::where('client_id', auth('client')->id())->where('shop_id', $request->shop_id)->where('product_id', $request->product_id)->first();

            if ($waiting) {
                $deleteWaiting = $waiting->delete();

                if ($deleteWaiting) {
                    return response()->json(['status' => true, 'msg' => 'تم حذف المنتج من قائمة الأنتظار لديك'], 200);
                }

                return response()->json(['status' => false, 'msg' => 'خطأ أثناء محاولة حذف منتج من قائمة الأنتظار'], 422);
            }

            return response()->json(['status' => false, 'msg' => 'المنتج غير موجود في قائمة الأنتظار'], 422);
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'msg' => $e->getMessage()], 422);
        }
    }
}
