<?php

namespace App\Http\Controllers\ApiV5;

use App\Card;
use App\Client;
use App\Color;
use App\Item;
use App\ItemColor;
use App\ItemSize;
use App\ItemType;
use App\ItemUnit;
use App\Rate;
use App\RequestDetails;
use App\RequestSettings;
use App\Size;
use Carbon\Carbon;
use function Couchbase\defaultDecoder;
use Illuminate\Http\Request;

use App\Unit;
use App\Favorite;
use App\ItemImage;
use App\Http\Requests;
use App\Http\Controllers\Controller;
use DB;

class CategoriesController extends BasicController
{


    public function allMainCats($shop_id)
    {

        $main_cats = ItemType::where('shop_id', $shop_id)
            ->where('category_id', 0)
            ->where('published',1)
            ->orderBy('display_order')
            ->select('id', 'name', 'upload_img')
            // ->select('id', 'name')
            ->get();

        foreach ($main_cats as $item) {
            $item->url = 'api/v1/cats/' . $item->id;

            if ($item->upload_img == "" or is_null($item->upload_img)) {
                $item->image = config('app.img_url') . 'logo.png';
            } else {
                $item->image = config('app.img_url') . $item->upload_img;
            }

        }

        return response()->json([
            'status' => true,
            'data' => $main_cats
        ]);

    }

    public function mainCatsDetails($shop_id)
    {
        $main_cats = ItemType::where('shop_id', $shop_id)
            ->where('category_id', 0) 
            ->where('published',1)
            ->orderBy('display_order')
            ->select('id', 'name')
            ->get();

        foreach ($main_cats as $item) {

            $sub_cats = ItemType::where('shop_id', $shop_id)
                ->where('category_id', $item->id)
                 ->where('published',1)
                ->orderBy('display_order')
                ->select('id', 'name')
                ->get();
            $item->sub_cats = $sub_cats;
        }

        return response()->json([

            'status' => true,
            'data' => $main_cats

        ]);
    }

    public function MainCatSubs($shop_id, $id)
    {
        $sub_cats = \DB::table('items_type')->where('shop_id', $shop_id)
            ->where('category_id', $id)
            ->where('published',1)
//            ->select('id', 'name', 'upload_img')
            ->orderBy('display_order')
            ->select('id', 'name', 'upload_img')
            ->get();

        foreach ($sub_cats as $item) {
            $item->url = 'api/v1/cats/' . $item->id;

            if ($item->upload_img == "" or is_null($item->upload_img)) {
                $item->image = config('app.img_url') . 'logo.png';
            } else {
                $item->image = config('app.img_url') . $item->upload_img;
            }

        }

        return response()->json([
            'status' => true,
            'data' => $sub_cats

        ]);
    }

    public function subCatItems($shop_id, $id, $sub)
    {

        $main = ItemType::where('shop_id', $shop_id)
            ->where('id', $id)
            ->where('category_id', 0)
             ->where('published',1)
            ->first();

        $subCat = ItemType::where('shop_id', $shop_id)
            ->where('id', $sub)
            ->where('category_id', $id)
             ->where('published',1)
            ->first();
                       
        if ($subCat) {
            $items = Item::join('units', 'items.unit_id', '=', 'units.id')
                ->where('items.shop_id', $shop_id)
                ->where('items.sale_unit', $subCat->id)
                ->where('items.available', 1)
                ->select('items.id', 'items.item_name', 'items.sale_price', 'items.sale_unit', 'items.img', 'items.unit_id',
                    'items.size_id', 'items.color_id', 'items.withDiscount', 'items.discount_percent',
                    'items.card_company_id')
                ->paginate(10);

            if (count($items) > 0) {
                            //  $items = $this->getItemsDataCollection($items , $shop_id);

                $response = [
                    'items' => $items,
                    'status' => true
                ];

                return response()->json($response, 200);
            } else {
                $response = [
                    'msg' => 'There is no items for this category',
                    'status' => false
                ];
                return response()->json($response, 200);
            }
        } else {
            $response = [
                'msg' => 'category error',
                'status' => false
            ];
            return response()->json($response, 200);
        }

    }

    public function getCatsIds($id)
    {
        $ids = [(int)$id];
        $cats = ItemType::where('category_id', $id)->where('published',1)->pluck('id');
        $cats = json_decode(json_encode($cats));
        $all = array_merge($ids, $cats);
        return $all;
    }

    public function internalCatItems(Request $request, $shop_id, $sub)
    {


        $subCat = ItemType::where('shop_id', $shop_id)
            ->where('id', $sub)
             ->where('published',1)
//            ->where('category_id', $sub)
            ->first();
        $online_units = $this->getAvailableUnits($shop_id);
        $subC = $request->cats;
        if ($sub == null && is_null($subC)) {
            $subC = ItemType::where('category_id', 0) ->where('published',1)->where('shop_id', $shop_id)->pluck('id');

        }
        $ids = [];
        if (is_null($subC) || sizeof($subC) <= 0) {
            $ids = $this->getCatsIds($sub);
        } else {

            for ($i = 0; $i <= sizeof($subC) - 1; $i++) {
                $n = $this->getCatsIds($subC[$i]);
                $ids = array_merge($n, $ids);
            }
            $ids = array_unique($ids);

        }

//            if ($subCat) {


        $items = Item::join('units', 'items.unit_id', '=', 'units.id')
            ->whereIn('items.sale_unit', $online_units)
            ->where('items.shop_id', $shop_id)
            ->whereIn('items.sale_unit', $ids)
            ->where('items.available', 1)
            ->where('online', 1)
            ->select(
                'items.id',
                'items.item_name',
                'items.sale_price',
                'items.sale_unit',
                'items.img',
                'items.unit_id',
                'items.size_id',
                'items.color_id',
                'items.withDiscount',
                'items.discount_percent',
                'items.quantity',
                'items.card_company_id',
                'items.vat_id',
                'items.vat_state',
                'items.shop_id'
            );
        if ($request->has('priceTo') && !in_array($request->priceTo, ['', null])) {
            $items = $items->where('sale_price', '<=', $request->priceTo);
        }
        if ($request->has('priceFrom') && !in_array($request->priceFrom, ['', null])) {
            $items = $items->where('sale_price', '>=', $request->priceFrom);
        }
        $items = $items->get();


        if (count($items) > 0) {
                          $items = $this->getItemsDataCollection($items , $shop_id);

            $response = [
                'items' => $items,
                'status' => true
            ];

            return response()->json($response, 200);
        } else {
            
            $response = [
                'msg' => 'áÇ ÊæÌÏ ãäÊÌÇÊ',
                'status' => false
            ];
            return response()->json($response, 200);
        }
//        } else {
//            $response = [
//                'msg' => 'ÞÓã ÛíÑ ãÚÑæÝ',
//                'status' => false
//            ];
//            return response()->json($response, 200);
//        }

    }

    private function cardCount($item)
    {

        $count = Card::where([
            'sale_id' => 0,
            'card_state' => 0,
            'shop_id' => $item->shop_id,
            'request_id' => 0,
            'item_id' => $item->id
        ])->count();

        return $count;
    }



    public function offerItems($shop_id)
    {
        $online_units = $this->getAvailableUnits($shop_id);

// dd("dsfd");
        //dd($online_units);

        $items = Item::join('units', 'items.unit_id', '=', 'units.id')
            ->whereIn('items.sale_unit', $online_units)
            ->where('items.shop_id', $shop_id)
            ->where('items.online', 1)
            ->where('items.available', 1)
            ->where('items.withDiscount', 1)
            ->where('items.discount_percent', '!=', 0)
            ->select(
                'items.id',
                'items.item_name',
                'items.sale_price',
                'items.sale_unit',
                'items.img',
                'items.unit_id',
                'items.size_id',
                'items.color_id',
                'items.withDiscount',
                'items.discount_percent',
                'items.quantity',
                'items.card_company_id',
                'items.vat_id',
                'items.vat_state',
                'items.shop_id'
            )
            ->paginate(10);


        if (count($items) > 0) {
            $items = $this->getItemsDataCollection($items , $shop_id);
            $items = $this->paginate($items , 10);

            $response = [
                'items' => $items,
                'status' => true
            ];
            return response()->json($response, 200);
        } else {
            $response = [
                'msg' => 'áÇ ÊæÌÏ ãäÊÌÇÊ',
                'status' => false
            ];
            return response()->json($response, 200);
        }


    }

    public function latestItems($shop_id)
    {
        $online_units = $this->getAvailableUnits($shop_id);

        $items = Item::join('units', 'items.unit_id', '=', 'units.id')
            ->whereIn('items.sale_unit', $online_units)
            ->where('items.shop_id', $shop_id)
            ->where('items.available', 1)
            ->where('online', 1)
            ->select(
                'items.id',
                'items.item_name',
                'items.sale_price',
                'items.sale_unit',
                'items.img',
                'items.unit_id',
                'items.size_id',
                'items.color_id',
                'items.withDiscount',
                'items.discount_percent',
                'items.details',
                'items.card_company_id',
                'items.quantity',
                'items.vat_id',
                'items.vat_state',
                'items.shop_id'
            )
            ->orderBy('id', 'DESC')
            ->take(10)
            ->get();


        if (count($items) > 0) {
            $items = $this->getItemsDataCollection($items , $shop_id);
            // $items = $this->paginate($items , 10);

            $response = [
                'items' => $items,
                'status' => true
            ];

            return response()->json($response, 200);
        } else {
            $response = [
                'msg' => 'لا يوجد اصناف',
                'status' => false
            ];
            return response()->json($response, 200);
        }

    }

    public function mostOrderedItems($shop_id)
    {
        /*
        select
            items.item_name, request_details.item_id,
            count(request_details.item_id) as count
            from request_details left JOIN items on items.id = request_details.item_id
        GROUP BY request_details.item_id
        ORDER BY count DESC
          */
        $online_units = $this->getAvailableUnits($shop_id);
      $items = Item::join('request_details' ,function ($j) use ($online_units){

          $j->on('request_details.item_id', '=', 'items.id');
          $j->whereIn('items.sale_unit', $online_units);
      }) ->join('units', 'request_details.unit_id', '=', 'units.id')
          ->select(
              \DB::raw('COUNT(request_details.item_id) as count'),
              'items.sale_price',
              'items.sale_unit',
              'items.img',
              'items.id',
              'items.unit_id',
              'items.withDiscount',
              'items.discount_percent',
              'items.item_name',
              'request_details.item_id',
              'items.card_company_id',

              'items.size_id',
              'items.color_id',
              'items.quantity',
              'items.vat_id',
              'items.vat_state',
              'items.shop_id'
          )
//            ->select( DB::raw('count(request_details.item_id) as count'), 'items.id', '', '')
          //DB::raw('COUNT(issue_subscriptions.issue_id) as followers')
          ->groupBy('request_details.item_id')
          ->orderBy('count', 'desc')
          ->where('request_details.shop_id', $shop_id)
          ->where('items.available', 1)
          ->where('items.online', 1)
          ->where('items.id', '!=', null)
          ->paginate(10);

//        $items = RequestDetails::leftJoin('items', function ($j) use ($online_units) {
//            $j->on('request_details.item_id', '=', 'items.id');
//            $j->whereIn('items.sale_unit', $online_units);
//        })
//            ->join('units', 'request_details.unit_id', '=', 'units.id')
//            ->select(
//                \DB::raw('COUNT(request_details.item_id) as count'),
//                'items.sale_price',
//                'items.sale_unit',
//                'items.img',
//                'items.id',
//                'items.unit_id',
//                'items.withDiscount',
//                'items.discount_percent',
//                'items.item_name',
//                'request_details.item_id',
//                'items.card_company_id',
//
//                'items.size_id',
//                'items.color_id',
//                'items.quantity',
//                'items.vat_id',
//                'items.vat_state',
//                'items.shop_id'
//            )
////            ->select( DB::raw('count(request_details.item_id) as count'), 'items.id', '', '')
//            //DB::raw('COUNT(issue_subscriptions.issue_id) as followers')
//            ->groupBy('request_details.item_id')
//            ->orderBy('count', 'desc')
//            ->where('request_details.shop_id', $shop_id)
//            ->where('items.available', 1)
//            ->where('items.online', 1)
//            ->where('items.id', '!=', null)
//            ->get();

        //return response()->json($items);
        /*
                $items = Item::where('shop_id', $shop_id)
                    ->whereIn('id', 1)
                    ->where('available', 1)
                    ->select('id', 'item_name', 'sale_price', 'sale_unit', 'img', 'unit_id', 'withDiscount', 'discount_percent')
                    ->paginate(10);
                */

        if (count($items) > 0) {
            $items = $this->getItemsDataCollection($items , $shop_id);
            $items = $this->paginate($items , 10);

            $response = [
                'items' => $items,
                'status' => true
            ];

            return response()->json($response, 200);
        } else {
            $response = [
                'msg' => 'áÇ ÊæÌÏ ãäÊÌÇÊ',
                'status' => false
            ];
            return response()->json($response, 200);
        }


    }

    public function itemDetails($shop_id, $id)
    {

//        $item = DB::table('items')
//            ->leftJoin('units', 'items.unit_id', '=', 'units.id')
//            ->where('items.shop_id', $shop_id)->where('items.id', $id)
//            ->select('items.item_name', 'items.sale_price', 'items.about',
//                'items.sale_unit', 'items.img', 'units.id', 'units.name')
//            ->first();


        $ci = Item::join('units', 'items.unit_id', '=', 'units.id')
            ->where('items.id', $id)
            ->where('items.shop_id', $shop_id)
            ->where('items.available', 1)
            ->where('items.online', 1)
            ->select(
                'items.id',
                'items.item_name',
                'items.sale_price',
                'items.sale_unit',
                'items.img',
                'items.unit_id',
                'items.size_id',
                'items.color_id',
                'items.withDiscount',
                'items.discount_percent',
                'items.details',
                'items.card_company_id',
                'items.size_id',
                'items.color_id',
                'items.quantity',
                'items.vat_id',
                'items.vat_state',
                'items.shop_id'
            )
            ->first();


        if ($ci) {


            $ci = $this->getItemData($ci, $shop_id);
            if($ci == false){
                             
                 $response = [
                    'status' => false,
                    'msg' => 'Item not found'
                ];
                return response()->json($response, 200);

                
            }
            $size_ = ItemSize::where('item_id', $ci->id)->pluck('size_id');

            $color_ = ItemColor::where('item_id', $ci->id)->pluck('color_id');

            $s_ids = json_decode(json_encode($size_));
            $c_ids = json_decode(json_encode($color_));

            $colors = Color::whereIn('id', $c_ids)->get();
            $sizes = Size::whereIn('id', $s_ids)->get();

            $avg_rate = Rate::where('shop_id',$shop_id)->where("item_id", $id)->avg('rate');
            $avg_rate = (int)$avg_rate;
            $ci->avg_rate = $avg_rate;
            $ci->rate = $avg_rate;
            $response = [
                'status' => true,
                'item' => $ci,
                'colors' => $colors,
                'sizes' => $sizes,
            ];
            return response()->json($response, 200);

        } else {
            $response = [
                'status' => false,
                'msg' => 'Item not found'
            ];
            return response()->json($response, 200);

        }

    }


    public function getAvailableUnits($shop_id)
    {

        $units = ItemType::where('shop_id', $shop_id)->where('category_id', 0)->where('published', 1)->pluck('id');
        $units = json_decode(json_encode($units));
        $arr = [];

        for ($i = 0; $i < sizeof($units); $i++) {
            $data = $this->getSubs($units[$i], $shop_id);
            $arr = array_merge($arr, $data);
        }
        $arr = array_merge($units, $arr);
        return $arr;
    }


    public function getsubs($id, $shop_id)
    {
        $a = [];
        $units = ItemType::where('shop_id', $shop_id)->where('category_id', $id)->where('published', 1)->pluck('id');
        $units = json_decode(json_encode($units));
        for ($i = 0; $i < sizeof($units); $i++) {
            $data = $this->getSubs($units[$i], $shop_id);
            $a = array_merge($a, $data);
        }
        $a = array_merge($units, $a);
        return $a;
    }




    public function rateItem(Request $request)
    {

        $this->validate($request, [
            'item_id' => 'required',
            'rate' => 'required',
            'shop_id' => 'required',
        ]);
        $client_id = $request->client_id;
        $api_token = $request->api_token;
        $client = Client::where('id', $client_id)->where('api_token', $api_token)->where('shop_id', $request->shop_id)->first();

        if ($client) {
            $item = Item::where('id', $request->item_id)->where('shop_id', $request->shop_id)->first();
            if ($item) {

                $rate = Rate::where('shop_id', $request->shop_id)->where('item_id', $item->id)->where('client_id', $client->id)->first();
                if ($rate) {
                    $rate->rate = $request->rate;
                    $rate->save();
                } else {
                    $rate = new Rate();
                    $rate->shop_id = $request->shop_id;
                    $rate->item_id = $item->id;
                    $rate->client_id = $client->id;
                    $rate->rate = $request->rate;
                    $rate->save();
                }

                return response()->json([
                    'status' => true,
                    'message' => 'تم التقييم بنجاح'
                ]);
            } else {
                $response = [
                    'status' => false,
                    'msg' => 'العنصر غير موجود'
                ];
                return response()->json($response, 200);

            }

        } else {
            $response = [
                'status' => false,
                'msg' => 'برجاء تسجيل الدخول اولا1'
            ];
            return response()->json($response, 200);

        }
    }




}
