<?php

namespace App\Http\Controllers;

use App\Badrshop;
use App\Client;
use App\Http\Traits\Helper;
use App\PasswordReset;
use Auth;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use App\Http\Requests;
use Illuminate\Http\Resources\Json\JsonResource;
use Redirect;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use App\AppClient;

class AuthController extends Controller
{
    use Helper;
    public function getLogin($id)
    {
        if (Auth::user()) {
            return back();
        } else {
            $shop = Badrshop::where('serial_id', $id)->firstOrFail();

            return view($shop->getViewPath('login'), compact('shop'));
        }
    }

    public function postLogin(Request $request, $id)
    {
        if (Auth::attempt(array('user_name' => $request->user_name, 'password' => $request->password))) {
            return redirect($id);
        } else {
            flash()->error('بيانات الدخول غير صحيحة');
            return back();
        }
    }

    public function register(Request $request)
    {
        $validator = Validator::make($request->all(),
            [
                'client_name' => 'required|max:50',
//                'email' => 'required',
                'tele' => 'required',
                'password' => 'required|confirmed',
            ], [
                'client_name.required' => 'يرجي ادخال الاسم',
//                'email.required' => 'يرجي ادخال البريد الالكتروني',
                'tele.required' => 'يرجي ادخال رقم الجوال',
                'tele.unique' => 'رقم الجوال موجود من قبل',
//                'email.unique' => 'البريد الالكتروني موجود من قبل',
            ]);
        if ($validator->fails()) {
            $errors = $validator->errors();
            $error_data = [];
            foreach ($errors->all() as $error) {
                array_push($error_data, $error);
            }
            $data = $error_data;
            $response = [
                'error' => $data,
                'status' => false
            ];
            return response()->json($response);
        }
        $isNew = false;
        $request->merge(['password' => md5($request->password)]);
        $appclient = AppClient::where('tele', $request->tele)->first();
        
        
        $player_id = $request->player_id;
        
        
        if (!$appclient) {
            $appclient = AppClient::Create($request->all());
            $isNew = true;
            $a_code =  rand(11111, 99999);
            $a_code = '123456';
            $appclient->verification_code = $a_code;
            $appclient->player_id = $player_id;
            
            $appclient->save();
            // send sms here
        }

        $client = Client::where('app_client_id', $appclient->id)
            ->where('shop_id', $request->shop_id)->first();
        if (!$client) {
            $client = new Client();
            $client->client_name = $request->client_name;
            $client->tele = $request->tele;
//            $client->email = $request->email;
            $client->password = $appclient->password;
            $client->shop_id = $request->shop_id;
            $client->app_client_id = $appclient->app_client_id;
            $client->api_token = Str::random($length = 60);
            
             $client->player_id = $player_id;

            $appclient->clients()->save($client);

            if ($appclient->status == 'active') {
                $response = [
                    'api_topken' => $client->api_token,
                    "data" => $appclient,
                    "msg" => "تم تسجيل بنجاح",
                    'status' => true,
                    'is_new' => false,
                ];
                return response()->json($response);
            } else {
                $response = [
                    'api_topken' => $client->api_token,
                    "data" => $appclient,
                    "msg" => "تم تسجيل بنجاح",
                    'status' => true,
                    'is_new' => true,
                ];
                return response()->json($response);
            }
            // if new client => resonse to activate
            // else [ no new client ] => response to login
        } else {
            // return false with already exists
            $response = [
                "msg" => "هذا العميل مسجل بالفعل",
                'status' => false,
            ];
            return response()->json($response);
        }
    }

    public function checkverificationcode(Request $request)
    {
        $appclient = AppClient::where('tele', $request->tele)->first();
        if (!$appclient)
            return $this->responseJson(0, 'رقم الجوال غير موجود');
        if ($appclient->verification_code === $request->verification_code) {
            $appclient->status = 'active';
            $appclient->save();
            $response = [
//                'api_token' => $appclient->api_token,
                "data" => $appclient,
                "msg" => "تم تفعيل الحساب بنجاح",
                'status' => true,
            ];
            return response()->json($response);
        } else {
            $response = [
                "msg" => "رمز التحقق غير صحيح",
                'status' => false,
            ];
            return response()->json($response);
        }
    }


    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'tele' => 'required',
            'password' => 'required',
            'shop_id' => 'required'
        ]);
        if ($validator->fails()) {
            $errors = $validator->errors();
            $error_data = [];
            foreach ($errors->all() as $error) {
                array_push($error_data, $error);
            }
            $data = $error_data;
            $response = [
                'error' => $data,
                'status' => false
            ];
            return response()->json($response);
        }
        $appclient = AppClient::where('tele', $request->tele)->first();

        $player_id = $request->player_id;
        
        if ($appclient) {
            $client = Client::where('app_client_id', $appclient->id)
                ->where('shop_id', $request->shop_id)->first();
            if ($appclient->status == 'active') {

                if (!$client) {
                    $client = new Client();
                    $client->client_name = $appclient->client_name;
                    $client->tele = $appclient->tele;
//                    $client->email = $appclient->email;
                    $client->password = $appclient->password;
                    $client->shop_id = $request->shop_id;
                    $client->app_client_id = $appclient->app_client_id;
                    // $client->api_token = Str::random($length = 60);
                    
                    $client->api_token = $player_id;
                    
                    
                    $appclient->clients()->save($client);

                    $response = [
                        'api_token' => $client->api_token,
                        "data" => $appclient,
                        "msg" => "تم تسجيل بنجاح",
                        'status' => true,
                        'is_new' => false,
                    ];
                    
                    
                    return response()->json($response);

                } else {
                    $password = md5($request->password);
                    if ($appclient->password === $password) {
                        
                        $appclient->player_id = $player_id;
                        
                        $appclient->save();
                        
                        
                        $response = [
                            'api_token' => $client->api_token,
                            "data" => $appclient,
                            "msg" => "تم تسجيل دخولك بنجاح",
                            'status' => true,
                        ];
                        return response()->json($response);

                    } else {
                        $response = [
                            "msg" => "كلمه المرور غير صحيحه",
                            'status' => false,
                        ];
                        return response()->json($response);
                    }
                }
            } else {
                $response = [
                    'account_status' => $appclient->status,
                    "msg" => "العميل غير مفعل",
                    'status' => false,
                ];
                return response()->json($response);
            }
        } else {
            $response = [
                "msg" => "رقم الجوال غير مسجل لدينا ",
                'status' => false,
            ];
            return response()->json($response);
        }
    }


    public function forgetPassword(Request $request)
    {
        $tele = $request->tele;
        $validator = Validator::make($request->all(), [
            'tele' => 'required',
        ]);
        if ($validator->fails()) {
            $errors = $validator->errors();
            $error_data = [];
            foreach ($errors->all() as $error) {
                array_push($error_data, $error);
            }
            $data = $error_data;
            $response = [
                'error' => $data,
                'status' => false
            ];
            return response()->json($response);
        }
        $appclient = AppClient::where('tele', $request->tele)->first();
        if (!$appclient) {
            return $this->responseJson(0, "العميل غير موجود");
        } else {
                $verify_code = rand(11111, 99999);
                PasswordReset::where('tele', $request->tele)->delete();
                $passwordReset = PasswordReset::create([
                    'tele' => $request->tele,
                    'verification_code' => $verify_code,
                    'created_at' => Carbon::now(),
                ]);

            // send sms here

            $response = [
                "msg" => "لقد أرسلنا الكود إلى رقم الجوال",
                'verify_code_for_test' => $verify_code,
                'status' => true,
            ];
            return response()->json($response);
        }
    }


    public function resetPassword(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'tele' => 'required',
            'password' => 'required|string|confirmed',
            'verification_code' => 'required|string',
        ]);
        if ($validator->fails()) {
            $errors = $validator->errors();
            $error_data = [];
            foreach ($errors->all() as $error) {
                array_push($error_data, $error);
            }
            $data = $error_data;
            $response = [
                'error' => $data,
                'status' => false
            ];
            return response()->json($response);
        }
        $passwordReset = PasswordReset::where([
            ['tele', $request->tele],
            ['verification_code', $request->verification_code],
        ])->first();
        if (!$passwordReset) {
            $response = [
                "msg" => "رمز التحقق غير صحيح",
                'status' => false,
            ];
            return response()->json($response);
        }
        $appclient = AppClient::where('tele', $passwordReset->tele)->first();
        if (!$appclient) {
            $response = [
                "msg" => 'العميل غير موجود',
                'status' => false,
            ];
            return response()->json($response);
        } else {

            $appclient->password = md5($request->password);
            $appclient->save();
            $clients = Client::where('app_client_id', $appclient->id)->get();
            foreach ($clients as $client){
                $client->password = md5($request->password);
                $client->save();
            }
            PasswordReset::where([
                ['verification_code', $request->verification_code],
                ['tele', $request->tele]
            ])->delete();

            $response = [
                "msg" => "تم اعاده تعين كلمه المرور بنجاح",
                "data" => $appclient,
                'status' => true,
            ];
            return response()->json($response);
        }
    }

    public function getClientData(Request $request){
        $shop_id = $request->shop_id;
        $app_client_id = $request->app_client_id;
        $appclient = AppClient::find($app_client_id);
        if (!$appclient) {
            $response = [
                "msg" => 'العميل غير موجود',
                'status' => false,
            ];
            return response()->json($response);
        } else {
            $client = Client::where('app_client_id', $appclient->id)->first();
            if(! $client){
                $client = new Client();
                $client->client_name = $appclient->client_name;
                $client->tele = $appclient->tele;
                $client->email = $appclient->email;
                $client->password = $appclient->password;
                $client->shop_id = $request->shop_id;
                $client->app_client_id = $appclient->app_client_id;
                $client->api_token = Str::random($length = 60);
                $appclient->clients()->save($client);

                $response = [
                    "api_token" => $client->api_token,
                    'status' => true,
                ];
                return response()->json($response);
            }else{
                $response = [
                    "api_token" => $client->api_token,
                    "data" => $appclient,
                    'status' => true,
                ];
                return response()->json($response);
            }
        }

    }

    public function profile(Request $request)
    {
        $clientApp = AppClient::find($request->app_client_id);
        if($clientApp){
            $rules = [
                'password' => 'confirmed',
                'email' =>'required|unique:app_clients,email,' . $clientApp->id,
                'tele' =>'required|unique:app_clients,tele,' . $clientApp->id,
                'client_name' => 'required',
            ];
            $messages = [
                'tele.unique' => 'رقم الهاتف موجود بالفعل',
                'tele.required' => 'يرجي ادخال رقم الهاتف',
                'email.required' => 'يرجي ادخال البريد الالكتروني',
                'email.unique' => 'البريد الالكتروني موجود بالفعل',
                'client_name.required' => 'يرجي ادخال اسم العميل',
            ];
            $validation = validator()->make($request->all(), $rules, $messages);
            if ($validation->fails()) {
                $errors = $validation->errors();
                $error_data = [];
                foreach ($errors->all() as $error) {
                    array_push($error_data, $error);
                }
                $data = $error_data;
                $response = [
                    'error' => $data,
                    'status' => false
                ];
                return response()->json($response);
            }
            $appclient = AppClient::find($request->app_client_id);
            if (!$appclient) {
                $response = [
                    "msg" => 'العميل غير موجود',
                    'status' => false,
                ];
                return response()->json($response);
             } else {
                $appclient->client_name = $request->client_name;
                $appclient->password = md5($request->password);
                $appclient->tele = $request->tele;
                $appclient->email = $request->email;
                $appclient->save();

                $clients = Client::where('app_client_id', $appclient->id)->get();
                foreach ($clients as $client) {
                    $client->client_name = $appclient->client_name;
                    $client->tele = $appclient->tele;
                    $client->email = $appclient->email;
                    $client->password = $appclient->password;
                    $client->save();
                }
                $response = [
                    "msg" => "تم تحديث الملف الشخصي بنجاح",
                    "data" => $appclient,
                    'status' => true,
                ];
                return response()->json($response);
            }
        }
        // no client (auth);
        $response = [
            "msg" => 'العميل غير موجود',
            'status' => false,
        ];
        return response()->json($response);
    }
}
