<?php

Route::group(['prefix' => 'api/v2/{shop_id}'], function () {
    // Users routes

    Route::get('/notifications', 'ApiV2\MainController@notifications');
    Route::post('get-my-notification', 'ApiV2\NotificationsController@getMyNotification');

    Route::get('/sliders', 'ApiV2\MainController@slider');
    Route::get('/pages', 'ApiV2\MainController@Pages');
    Route::get('/page/{id}', 'ApiV2\MainController@PageDetails');

    Route::group(['prefix' => 'users'], function () {
        Route::post('login', 'ApiV2\UsersController@authenticate');
        Route::post('get-location', 'ApiV2\UsersController@getLocation');
        Route::get('{user_id}/orders', 'ApiV2\UsersController@getUserOrders');
        Route::post('{user_id}/shortcomings', 'ApiV2\UsersController@shortcomings');
        Route::get('{user_id}/orders/{order_id}', 'ApiV2\UsersController@orderDetails');
        Route::get('{user_id}/new-orders', 'ApiV2\UsersController@getUserNewOrders');
        Route::post('{user_id}/orders/{order_id}/confirm', 'ApiV2\UsersController@confirmOrder');
        Route::post('{user_id}/search-for-client-order', 'ApiV2\UsersController@searchForClient');
        Route::post('{user_id}/stock', 'ApiV2\UsersController@userStock');
        Route::post('{user_id}/order-cancel', 'ApiV2\UsersController@orderCancel');
        Route::get('user-city/{city_id}', 'ApiV2\UsersController@getUsersOfCity');
        Route::post('switch-order', 'ApiV2\UsersController@switchOrder');
        // This route is responsible for rating the client
        Route::post('rate-client', 'ApiV2\UsersController@rateClient');
        Route::post('rate-item', 'ApiV2\CategoriesController@rateItem');
        // Users chat
        Route::post('messages', 'ApiV2\ChatController@getUserMessages');
        Route::post('get-messages', 'ApiV2\ChatController@userRequestMessages');
        Route::post('send-message', 'ApiV2\ChatController@userSendMessage');
        // Users tokens
        Route::post('register-token', 'ApiV2\ChatController@userRegisterToken');
        Route::post('remove-token', 'ApiV2\ChatController@userRemoveToken');
    });

    Route::group(['prefix' => 'clients'], function () {
        Route::post('add-to-favorites', 'ApiV2\FavoritesController@add');
        Route::post('remove-from-favorites', 'ApiV2\FavoritesController@remove');
        Route::post('get-client-favorites', 'ApiV2\FavoritesController@getClientFavorites');
        Route::post('/create', 'ApiV2\ClientsController@create');
        Route::post('login', 'ApiV2\ClientsController@authenticate');
        // Get client orders
        Route::post('client-orders', 'ApiV2\ClientsController@getOrders');
        // Client activation
        Route::post('active', 'ApiV2\ClientsController@activeClient');
        // Client chat
        Route::post('messages', 'ApiV2\ChatController@getClientMessages');
        Route::post('get-messages', 'ApiV2\ChatController@clientRequestMessages');
        Route::post('send-message', 'ApiV2\ChatController@clientSendMessage');
        // Client tokens
        Route::post('register-token', 'ApiV2\ChatController@clientRegisterToken');
        Route::post('remove-token', 'ApiV2\ChatController@clientRemoveToken');
        Route::post('request-code', 'ApiV2\ClientsController@requestActivationCode');
    });

    Route::get('/request-settings', 'ApiV2\MainController@requestSettings');

    // Categories
    Route::get('/cats', 'ApiV2\CategoriesController@allMainCats');
    Route::get('/cats-subs', 'ApiV2\CategoriesController@mainCatsDetails');
    Route::get('cats/{cat_id}', 'ApiV2\CategoriesController@MainCatSubs');
    Route::get('cats/{cat_id}/subcats/{sub_id}', 'ApiV2\CategoriesController@subCatItems');

    #use internal category to get data
    Route::get('cat-items/{cat_id}', 'ApiV2\CategoriesController@internalCatItems');

    // simple search
    Route::post('search', 'ApiV2\MainController@search');

    // Items Prices
    Route::post('items/prices', 'ApiV2\ItemController@getPrices')->name('item-prices');

    // Item details
    Route::get('items/{id}', 'ApiV2\CategoriesController@itemDetails')->name('item-details');
    Route::get('offer-items', 'ApiV2\CategoriesController@offerItems');
    Route::get('latest-items', 'ApiV2\CategoriesController@latestItems');
    Route::get('most-ordered-items', 'ApiV2\CategoriesController@mostOrderedItems');
    // Provinces
    Route::get('provinces', 'ApiV2\ProvincesController@getAllProvinces');
    Route::get('provinces/{p_id}', 'ApiV2\ProvincesController@getCitiesOfProvince');
    // Orders
    //    Route::post('orders/new', 'ApiV2\OrdersController@create');
    Route::post('orders/new', 'ApiV2\NewOrderController@saveRequest');
    Route::post('orders/request-img', 'ApiV2\NewOrderController@saveRequesImg');

    Route::post('bill/new', 'ApiV2\OrdersController@saveBillCall');

    Route::post('hang-order', 'ApiV2\OrdersController@hangOrder');

    Route::post('save-transaction', 'ApiV2\RequestController@saveFortId');

    Route::post('stand-alone', 'ApiV2\MainController@fort')->middleware('auth:client');
    Route::post('get_distance_price', 'ApiV2\NewOrderController@getDistancePrice');

    // Password resets
    /*
     Route::post('password/email', 'Auth\ForgotPasswordController@getResetToken');
     Route::post('password/reset', 'Auth\ResetPasswordController@reset');
 */

    Route::post('email-code', 'ApiV2\ClientsController@sendCode');
    Route::post('reset-password', 'ApiV2\ClientsController@resetClientPassword');
     // المدن 
    Route::get('get-cities', 'ApiV2\MainController@getCities');
    // المناطق
    // Route::post('get-areas', 'ApiV2\MainController@getAreas');
    //hatem
    Route::get('clients/profile', 'ApiV2\MainController@profileView');
    Route::post('profile', 'ApiV2\MainController@profileSave');
    Route::get('user-profile', 'ApiV2\MainController@userProfileView');
    Route::post('user-profile', 'ApiV2\MainController@userProfileSave');
    Route::get('user-stock', 'ApiV2\MainController@userStock');
    Route::get('short-comings', 'ApiV2\MainController@shortcomings');

    Route::get('requests', 'ApiV2\RequestController@requests');
    Route::post('request-start-move', 'ApiV2\RequestController@startMove');
    
    
    Route::get('edit-order/{request_id}', 'ApiV2\RequestController@editOrder');
    
    Route::get('request-details/{request_id}', 'ApiV2\RequestController@requestDetails');
    Route::post('request-cancel', 'ApiV2\RequestController@requestCancel');
    Route::post('request-rate', 'ApiV2\RequestController@requestRate');
    Route::post('request-details-rate', 'ApiV2\RequestController@requestDetailsRate');
    Route::get('about-us', 'ApiV2\MainController@aboutUs');
    Route::get('terms-of-use', 'ApiV2\MainController@termsOfUse');
    Route::get('contact-us', 'ApiV2\MainController@contactUs');
    Route::post('contact-us', 'ApiV2\MainController@postContactUs');


    Route::get('item-colors', 'ApiV2\MainController@getItemsColors');
    Route::get('item-sizes', 'ApiV2\MainController@getItemsSizes');

    Route::get('get-regions', 'ApiV2\ProvincesController@getRegions');
    Route::get('get-region-cities', 'ApiV2\ProvincesController@getRegionCities');

    Route::get('chat', function ($id) {
        return response()->json([
            'status' => true,
            //            'url' => urldecode('https://tawk.to/chat/5eb7b0d3967ae56c52186410/default/?$_tawk_popout=true')
            'url' => url('api/v1/' . $id . '/call')

        ]);
    });

    Route::get('client-data', function ($id, Request $request) {

        $settings = \App\RequestSettings::where('shop_id', $id)->first();

        $client = auth()->guard('client')->user();
        if ($client) {
            $regions = \DB::table('regions')->selectRaw('id, name_ar, name_en')->get();
            $city = \DB::table('charge_cities')->find($client->city_id);
            $region_id = 0;
            $cities = \DB::table('charge_cities')->where('region_id', $regions[0]->id ?? 0)->selectRaw('id, name_ar, name_en')->get();

            if ($city) {
                $region_id = $city->region_id;
                $cities = \DB::table('charge_cities')->where('region_id', $region_id)->selectRaw('id, name_ar, name_en')->get();
            }


            return response()->json([
                'status' => true,
                'tele' => $client->tele,
                'city_id' => $client->city_id,
                'region_id' => $region_id,
                'regions' => $regions,
                'cities' => $cities,
                'lon' => '21.21212',
                'lat' => '39.18945880',
                'epay' => (bool) $settings->epay ?? true,
                'delivery' => (bool) $settings->delivery ?? false,
                'bank' => (bool) $settings->bank ?? true,
            ]);
        } else {
            return response()->json([
                'status' => false,
                'msg' => 'no user'
            ]);
        }
    });


    Route::post('check-coupon', 'ApiV2\NewOrderController@checkCoupon');
    Route::get('delete-account', 'ApiV2\ClientsController@deleteAccount');


    Route::get('call', function () {
        return view('api/chat');
    });
});


?>