<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class Item extends Model
{
    protected $casts = [ 'id' => 'integer' ];
    const SAVE=0;
    const UPDATE=1;

    public function category()
    {
        return $this->belongsTo('App\ItemType', 'sale_unit');
    }

    public function mainUnit() {
        return $this->belongsTo(Unit::class, 'unit_id');
    }

    public function images()
    {
        return $this->hasMany('App\ItemImage');
    }

    public function calcQuantity() {
        return RequestDetails::join('requests', 'requests.id', '=', 'request_details.request_id')
            ->where('request_details.item_id', $this->attributes['item_id'])
            ->where('request_details.shop_id', $this->attributes['shop_id'])
            ->whereNotIn('requests.status', ['0'])
            ->sum('quantity');
    }

//    public function shop()
//    {
//        //return $this->belongsTo('App\Badrshop','shop_id','serial_id');
//    }
    /*
        public function getImgAttribute($img)
        {
            if ($img == "")
            {
                return "images/img.jpg";
            }
            $url = str_replace('https:/www.badrforsales.com/online/','',$img);
            $url = str_replace('https://www.badrforsales.com/online/','',$img);
            $url = str_replace('https://badrforsales.com/online/','',$img);
            return $url;
        }

        public function getSalePriceAttribute($price)
        {
            if (count($this->shop) > 0)
            {
                return number_format($price,$this->shop->decimal_num_price);
            }
            return number_format($price,2);
        }


    */

    public function getImg($shop_id)
    {

        // $logo = config('app.badrshop_url') . "images/img.jpg";

        $logo = config('app.badrshop_url') . "images/app-logo.png";
//   return config('app.badrshop_url') . $this->img;
        if (in_array($this->img, ['', null])) {
            return $logo;
        } else {
            /* if (file_exists('admin/' . $this->img)) {

                 return config('app.badrshop_url') . $this->img;
             } elseif (file_exists('admin/upload/' . $this->img)) {
           */

            if (strpos( $this->img, 'upload/') !== false) {
                if(strpos( $this->img, 'upload/Invalid file format..') !== false){
                    return $logo;
                }
                // return config('my-connections.'.$shop_id.'.base_url') . $this->img;
                return config('app.badrshop_url') . $this->img;
            }
            
            if(strpos( $this->img, 'images/img.jpg') !== false){
                return $logo;
            }
            
            // return config('my-connections.'.$shop_id.'.base_url') . 'upload/' . $this->img;
            return config('app.badrshop_url') . $this->img;
            /*   } else {
                   return $logo;
               }*/
        }
    }

    public function getCards($bill_id)
    {
        $cards_query = Card::where([
            'shop_id' => $this->shop_id,
            'item_id' => $this->id,
            'sale_id' => $bill_id,
            'card_state' => 1
        ])->pluck('card_number');
        $data = [];
        $cards_enc = json_decode(json_encode($cards_query));
        if (sizeof($cards_enc) > 0) {
            foreach ($cards_query as $item) {
                $data[] = $this->decrypt($item);
            }
        }
        return $data;
    }

    private function decrypt($encrypted)
    {
        $password = '3sc3RLrpd1LBVFTsdFVaadjsdXFS';
        $method = 'aes-256-cbc';
        $key = substr(hash('sha256', $password, true), 0, 512);

        $iv = chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0);

        $decrypted = openssl_decrypt(base64_decode($encrypted), $method, $key, OPENSSL_RAW_DATA, $iv);

        $data = explode('--', $decrypted);
        $decrypt = $data[1];
        return $decrypt;
    }

    public function cardId() {
        $card_company_id = $this->card_company_id;
        if (in_array($card_company_id, ['0', '', null])) {
            return null;
        }
        return $card_company_id;
    }

    public function itemPrice($unit_id, $quantity, $action_type, $item_details, $shop_id) {
        if ($this->unit_id == $unit_id) {
            // if($action_type == self::UPDATE) {
            //     if($item_details != null) {
            //         if($item_details->quantity == $quantity && $item_details->unit_id == $unit_id) {
            //             return $item_details->sale_price;
            //         } else {
            //             return $this->sale_price;
            //         }
            //     } else {
            //         return $this->sale_price;
            //     }
            // } else {
            //     return $this->sale_price;
            // }
            return $this->sale_price;
        } else {
            $unit = ItemUnit::where('shop_id', $shop_id)->where('unit_id', $unit_id)->where('item_id', $this->id)->first();
            return $unit->unit_price;
        }
    }

    public function itemPriceAfterDiscount($price, $discount_type = 'percent') {
        if ($this->withDiscount) {
                if ($discount_type == 'percent') {
                    return $price - ($price * ($this->discount_percent / 100));
                } else {
                    return $price - $this->discount_percent;
                }
            
        } else {
            return $price;
        }
    }

    public function itemPriceAfterVat($price_after_discount, $bill_adds) {
        if ($this->vat_state == 2) { // شامل
            if ($v = $bill_adds->where('id', $this->vat_id)) {
                if ($v->check_addition == 0) {
                    return $price_after_discount / (1 + $v->addition_value / 100);
                }
            }
        } else if ($this->vat_state == 1) { // غير شامل
            $vat = 0;
            foreach ($bill_adds as $add) {
                if ($add->check_addition) {
                    $item_vat = $add->addition_value;
                    $vat += $item_vat;
                } else {
                    $item_vat = $price_after_discount * $add->addition_value / 100;
                    $vat += $item_vat;
                }
            }
            return $price_after_discount + $vat;
        } else { // معفي
            return $price_after_discount;
        }
    }

}
