<?php

namespace App\Providers;

use Illuminate\Support\ServiceProvider;
use Validator;
use Illuminate\Support\Facades\Config;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        Validator::extend('ksa_phone', function($attribute, $value, $parameters, $validator) {
            return preg_match('/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/', $value);
        });
        
        \Log::info(request()->getUri());
        
        
        Validator::extend('unique_shop_item', function ($attribute, $value, $parameters) {
            list($table, $field, $field2, $field2Value) = $parameters;
            return \DB::table($table)->where($field, $value)
            ->where($field2, $field2Value)
            ->count() == 0;
        });
        
        Validator::extend('unique_shop_item_update', function ($attribute, $value, $parameters) {
            list($table, $field, $field2, $field2Value, $field3, $field3Value) = $parameters;
            return \DB::table($table)->where($field, $value)
            ->where($field2, $field2Value)
            ->where($field3, '!=', $field3Value)
            ->count() == 0;
        });
        

        $shop_id = request()->segment(3);
        
        $mode = 'live';
        if(request()->header('env-type') == 'test'){
            $mode = 'test';
        }
        
        $connection = config('my-connections.' . $mode .  '.' . $shop_id);

        if(! is_array($connection)){
            abort(404);
        }   

        Config::set("database.connections.mysql.host",$connection['host'] );
        Config::set("database.connections.mysql.post",$connection['port'] );
        Config::set("database.connections.mysql.database",$connection['database'] );
        Config::set("database.connections.mysql.username",$connection['username'] );
        Config::set("database.connections.mysql.password", $connection['password']);
        
        Config::set("app.base_url", $connection['base_url']);
        Config::set("app.img_url", $connection['img_url']);
        Config::set("app.badrshop_url", $connection['badrshop_url']);

        Config::clearResolvedInstances();


    }

    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        require_once __DIR__ . '/../Http/Rate.php';
        require_once __DIR__ . '/../Http/helpers/helper.php';
        // require_once __DIR__ . '/../../codegenertate.php';
    }
}
